@@ -14,9 +14,11 @@ import FastString
 
 import Data.Data
 
+-- | Haskell Documentation String
 newtype HsDocString = HsDocString FastString
-  deriving (Eq, Show, Data, Typeable)
+  deriving (Eq, Show, Data)
 
+-- | Located Haskell Documentation String
 type LHsDocString = Located HsDocString
 
 instance Outputable HsDocString where
