@@ -1,3 +1,5 @@
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
+
 module Llvm.MetaData where
 
 import Llvm.Types
@@ -9,7 +11,7 @@ import Outputable
 -- following design:
 -- * Metadata can be constructed in a few different ways (See below).
 -- * After which it can either be attached to LLVM statements to pass along
--- extra information to the optimizer and code generator OR specificially named
+-- extra information to the optimizer and code generator OR specifically named
 -- metadata has an affect on the whole module (i.e., linking behaviour).
 --
 --
@@ -55,18 +57,26 @@ import Outputable
 --     !llvm.module.linkage = !{ !0, !1 }
 --
 
+-- | A reference to an un-named metadata node.
+newtype MetaId = MetaId Int
+               deriving (Eq, Ord, Enum)
+
+instance Outputable MetaId where
+    ppr (MetaId n) = char '!' <> int n
+
 -- | LLVM metadata expressions
-data MetaExpr = MetaStr LMString
-              | MetaNode Int
-              | MetaVar LlvmVar
+data MetaExpr = MetaStr !LMString
+              | MetaNode !MetaId
+              | MetaVar !LlvmVar
               | MetaStruct [MetaExpr]
               deriving (Eq)
 
 instance Outputable MetaExpr where
-  ppr (MetaStr    s ) = text "!\"" <> ftext s <> char '"'
-  ppr (MetaNode   n ) = text "!" <> int n
+  ppr (MetaVar (LMLitVar (LMNullLit _))) = text "null"
+  ppr (MetaStr    s ) = char '!' <> doubleQuotes (ftext s)
+  ppr (MetaNode   n ) = ppr n
   ppr (MetaVar    v ) = ppr v
-  ppr (MetaStruct es) = text "!{ " <> ppCommaJoin es <> char '}'
+  ppr (MetaStruct es) = char '!' <> braces (ppCommaJoin es)
 
 -- | Associates some metadata with a specific label for attaching to an
 -- instruction.
@@ -77,7 +87,7 @@ data MetaAnnot = MetaAnnot LMString MetaExpr
 data MetaDecl
     -- | Named metadata. Only used for communicating module information to
     -- LLVM. ('!name = !{ [!<n>] }' form).
-    = MetaNamed LMString [Int]
+    = MetaNamed !LMString [MetaId]
     -- | Metadata node declaration.
     -- ('!0 = metadata !{ <metadata expression> }' form).
-    | MetaUnamed Int MetaExpr
+    | MetaUnnamed !MetaId !MetaExpr
