@@ -42,6 +42,7 @@ import Format
 import TargetReg
 
 import BlockId
+import Hoopl
 import CLabel           ( CLabel, mkAsmTempLabel )
 import Debug
 import FastString       ( FastString )
@@ -52,9 +53,6 @@ import DynFlags
 import Module
 
 import Control.Monad    ( liftM, ap )
-#if __GLASGOW_HASKELL__ < 709
-import Control.Applicative ( Applicative(..) )
-#endif
 
 import Compiler.Hoopl   ( LabelMap, Label )
 
@@ -87,7 +85,6 @@ initNat :: NatM_State -> NatM a -> (a, NatM_State)
 initNat init_st m
         = case unNat m init_st of { (r,st) -> (r,st) }
 
-
 instance Functor NatM where
       fmap = liftM
 
@@ -97,8 +94,15 @@ instance Applicative NatM where
 
 instance Monad NatM where
   (>>=) = thenNat
-  return = pure
 
+instance MonadUnique NatM where
+  getUniqueSupplyM = NatM $ \st ->
+      case splitUniqSupply (natm_us st) of
+          (us1, us2) -> (us1, st {natm_us = us2})
+
+  getUniqueM = NatM $ \st ->
+      case takeUniqFromSupply (natm_us st) of
+          (uniq, us') -> (uniq, st {natm_us = us'})
 
 thenNat :: NatM a -> (a -> NatM b) -> NatM b
 thenNat expr cont
