@@ -22,7 +22,7 @@ import FastString
 import Binary
 import Outputable
 import Module
-import BasicTypes ( SourceText )
+import BasicTypes ( SourceText, pprWithSourceText )
 
 import Data.Char
 import Data.Data
@@ -37,7 +37,6 @@ import Data.Data
 
 newtype ForeignCall = CCall CCallSpec
   deriving Eq
-  {-! derive: Binary !-}
 
 isSafeForeignCall :: ForeignCall -> Bool
 isSafeForeignCall (CCall (CCallSpec _ _ safe)) = playSafe safe
@@ -62,9 +61,8 @@ data Safety
 
   | PlayRisky           -- None of the above can happen; the call will return
                         -- without interacting with the runtime system at all
-  deriving ( Eq, Show, Data, Typeable )
+  deriving ( Eq, Show, Data )
         -- Show used just for Show Lex.Token, I think
-  {-! derive: Binary !-}
 
 instance Outputable Safety where
   ppr PlaySafe = text "safe"
@@ -94,15 +92,13 @@ data CExportSpec
                                 -- See note [Pragma source text] in BasicTypes
         CLabelString            -- C Name of exported function
         CCallConv
-  deriving (Data, Typeable)
-  {-! derive: Binary !-}
+  deriving Data
 
 data CCallSpec
   =  CCallSpec  CCallTarget     -- What to call
                 CCallConv       -- Calling convention to use.
                 Safety
   deriving( Eq )
-  {-! derive: Binary !-}
 
 -- The call target:
 
@@ -129,8 +125,7 @@ data CCallTarget
                                         -- allowed in CAPI imports
   | DynamicTarget
 
-  deriving( Eq, Data, Typeable )
-  {-! derive: Binary !-}
+  deriving( Eq, Data )
 
 isDynamicTarget :: CCallTarget -> Bool
 isDynamicTarget DynamicTarget = True
@@ -150,8 +145,7 @@ See: http://www.programmersheaven.com/2/Calling-conventions
 
 -- any changes here should be replicated in  the CallConv type in template haskell
 data CCallConv = CCallConv | CApiConv | StdCallConv | PrimCallConv | JavaScriptCallConv
-  deriving (Eq, Data, Typeable)
-  {-! derive: Binary !-}
+  deriving (Eq, Data)
 
 instance Outputable CCallConv where
   ppr StdCallConv = text "stdcall"
@@ -209,14 +203,14 @@ instance Outputable CCallSpec where
       gc_suf | playSafe safety = text "_GC"
              | otherwise       = empty
 
-      ppr_fun (StaticTarget _ fn mPkgId isFun)
+      ppr_fun (StaticTarget st _fn mPkgId isFun)
         = text (if isFun then "__pkg_ccall"
                          else "__pkg_ccall_value")
        <> gc_suf
        <+> (case mPkgId of
             Nothing -> empty
             Just pkgId -> ppr pkgId)
-       <+> pprCLabelString fn
+       <+> (pprWithSourceText st empty)
 
       ppr_fun DynamicTarget
         = text "__dyn_ccall" <> gc_suf <+> text "\"\""
@@ -224,10 +218,10 @@ instance Outputable CCallSpec where
 -- The filename for a C header file
 -- Note [Pragma source text] in BasicTypes
 data Header = Header SourceText FastString
-    deriving (Eq, Data, Typeable)
+    deriving (Eq, Data)
 
 instance Outputable Header where
-    ppr (Header _ h) = quotes $ ppr h
+    ppr (Header st h) = pprWithSourceText st (doubleQuotes $ ppr h)
 
 -- | A C type, used in CAPI FFI calls
 --
@@ -239,10 +233,12 @@ instance Outputable Header where
 data CType = CType SourceText -- Note [Pragma source text] in BasicTypes
                    (Maybe Header) -- header to include for this type
                    (SourceText,FastString) -- the type itself
-    deriving (Eq, Data, Typeable)
+    deriving (Eq, Data)
 
 instance Outputable CType where
-    ppr (CType _ mh (_,ct)) = hDoc <+> ftext ct
+    ppr (CType stp mh (stct,ct))
+      = pprWithSourceText stp (text "{-# CTYPE") <+> hDoc
+        <+> pprWithSourceText stct (doubleQuotes (ftext ct)) <+> text "#-}"
         where hDoc = case mh of
                      Nothing -> empty
                      Just h -> ppr h
@@ -255,7 +251,6 @@ instance Outputable CType where
 ************************************************************************
 -}
 
-{-* Generated by DrIFT-v1.0 : Look, but Don't Touch. *-}
 instance Binary ForeignCall where
     put_ bh (CCall aa) = put_ bh aa
     get bh = do aa <- get bh; return (CCall aa)
