@@ -25,11 +25,11 @@ module CoAxiom (
 
        Role(..), fsFromRole,
 
-       CoAxiomRule(..), Eqn,
+       CoAxiomRule(..), TypeEqn,
        BuiltInSynFamily(..), trivialBuiltInFamily
        ) where
 
-import {-# SOURCE #-} TyCoRep ( Type )
+import {-# SOURCE #-} TyCoRep ( Type, pprType )
 import {-# SOURCE #-} TyCon ( TyCon )
 import Outputable
 import FastString
@@ -124,16 +124,13 @@ type BranchIndex = Int  -- The index of the branch in the list of branches
 -- promoted data type
 data BranchFlag = Branched | Unbranched
 type Branched = 'Branched
-deriving instance Typeable 'Branched
 type Unbranched = 'Unbranched
-deriving instance Typeable 'Unbranched
 -- By using type synonyms for the promoted constructors, we avoid needing
 -- DataKinds and the promotion quote in client modules. This also means that
 -- we don't need to export the term-level constructors, which should never be used.
 
 newtype Branches (br :: BranchFlag)
   = MkBranches { unMkBranches :: Array BranchIndex CoAxBranch }
-  deriving Typeable
 type role Branches nominal
 
 manyBranches :: [CoAxBranch] -> Branches Branched
@@ -216,7 +213,6 @@ data CoAxiom br
                                       -- See Note [Implicit axioms]
          -- INVARIANT: co_ax_implicit == True implies length co_ax_branches == 1.
     }
-  deriving Typeable
 
 data CoAxBranch
   = CoAxBranch
@@ -235,7 +231,7 @@ data CoAxBranch
     , cab_incomps  :: [CoAxBranch]  -- The previous incompatible branches
                                     -- See Note [Storing compatibility]
     }
-  deriving ( Data.Data, Data.Typeable )
+  deriving Data.Data
 
 toBranchedAxiom :: CoAxiom br -> CoAxiom Branched
 toBranchedAxiom (CoAxiom unique name role tc branches implicit)
@@ -385,15 +381,8 @@ See also Note [Implicit TyThings] in HscTypes
 -}
 
 instance Eq (CoAxiom br) where
-    a == b = case (a `compare` b) of { EQ -> True;   _ -> False }
-    a /= b = case (a `compare` b) of { EQ -> False;  _ -> True  }
-
-instance Ord (CoAxiom br) where
-    a <= b = case (a `compare` b) of { LT -> True;  EQ -> True;  GT -> False }
-    a <  b = case (a `compare` b) of { LT -> True;  EQ -> False; GT -> False }
-    a >= b = case (a `compare` b) of { LT -> False; EQ -> True;  GT -> True  }
-    a >  b = case (a `compare` b) of { LT -> False; EQ -> False; GT -> True  }
-    compare a b = getUnique a `compare` getUnique b
+    a == b = getUnique a == getUnique b
+    a /= b = getUnique a /= getUnique b
 
 instance Uniquable (CoAxiom br) where
     getUnique = co_ax_unique
@@ -414,8 +403,9 @@ instance Outputable CoAxBranch where
   ppr (CoAxBranch { cab_loc = loc
                   , cab_lhs = lhs
                   , cab_rhs = rhs }) =
-    text "CoAxBranch" <+> parens (ppr loc) <> colon <+> ppr lhs <+>
-    text "=>" <+> ppr rhs
+    text "CoAxBranch" <+> parens (ppr loc) <> colon
+      <+> brackets (fsep (punctuate comma (map pprType lhs)))
+      <+> text "=>" <+> pprType rhs
 
 {-
 ************************************************************************
@@ -429,8 +419,11 @@ Roles are defined here to avoid circular dependencies.
 
 -- See Note [Roles] in Coercion
 -- defined here to avoid cyclic dependency with Coercion
+--
+-- Order of constructors matters: the Ord instance coincides with the *super*typing
+-- relation on roles.
 data Role = Nominal | Representational | Phantom
-  deriving (Eq, Ord, Data.Data, Data.Typeable)
+  deriving (Eq, Ord, Data.Data)
 
 -- These names are slurped into the parser code. Changing these strings
 -- will change the **surface syntax** that GHC accepts! If you want to
@@ -474,19 +467,19 @@ as `CoAxiom` is the special case when there are no assumptions.
 -}
 
 -- | A more explicit representation for `t1 ~ t2`.
-type Eqn = Pair Type
+type TypeEqn = Pair Type
 
 -- | For now, we work only with nominal equality.
 data CoAxiomRule = CoAxiomRule
   { coaxrName      :: FastString
   , coaxrAsmpRoles :: [Role]    -- roles of parameter equations
   , coaxrRole      :: Role      -- role of resulting equation
-  , coaxrProves    :: [Eqn] -> Maybe Eqn
+  , coaxrProves    :: [TypeEqn] -> Maybe TypeEqn
         -- ^ coaxrProves returns @Nothing@ when it doesn't like
         -- the supplied arguments.  When this happens in a coercion
         -- that means that the coercion is ill-formed, and Core Lint
         -- checks for that.
-  } deriving Typeable
+  }
 
 instance Data.Data CoAxiomRule where
   -- don't traverse?
@@ -510,9 +503,9 @@ instance Outputable CoAxiomRule where
 -- Type checking of built-in families
 data BuiltInSynFamily = BuiltInSynFamily
   { sfMatchFam      :: [Type] -> Maybe (CoAxiomRule, [Type], Type)
-  , sfInteractTop   :: [Type] -> Type -> [Eqn]
+  , sfInteractTop   :: [Type] -> Type -> [TypeEqn]
   , sfInteractInert :: [Type] -> Type ->
-                       [Type] -> Type -> [Eqn]
+                       [Type] -> Type -> [TypeEqn]
   }
 
 -- Provides default implementations that do nothing.
