@@ -45,7 +45,7 @@ colorGraph
 
         -> ( Graph k cls color          -- the colored graph.
            , UniqSet k                  -- the set of nodes that we couldn't find a color for.
-           , UniqFM  k )                -- map of regs (r1 -> r2) that were coaleced
+           , UniqFM  k )                -- map of regs (r1 -> r2) that were coalesced
                                         --       r1 should be replaced by r2 in the source
 
 colorGraph iterative spinCount colors triv spill graph0
@@ -184,7 +184,7 @@ colorScan_spin iterative triv spill graph
                 kksCoalesce
 
         -- Coalesce:
-        --      If we're doing iterative coalescing and no triv nodes are avaliable
+        --      If we're doing iterative coalescing and no triv nodes are available
         --      then it's time for a coalescing pass.
         | iterative
         = case coalesceGraph False triv graph of
@@ -309,8 +309,9 @@ selectColor colors graph u
         Just nsConflicts
                         = sequence
                         $ map (lookupNode graph)
-                        $ uniqSetToList
+                        $ nonDetEltsUniqSet
                         $ nodeConflicts node
+                        -- See Note [Unique Determinism and code generation]
 
         colors_conflict = mkUniqSet
                         $ catMaybes
@@ -355,7 +356,8 @@ selectColor colors graph u
 
                 -- it wasn't a preference, but it was still ok
                 | not $ isEmptyUniqSet colors_ok
-                , c : _         <- uniqSetToList colors_ok
+                , c : _         <- nonDetEltsUniqSet colors_ok
+                -- See Note [Unique Determinism and code generation]
                 = Just c
 
                 -- no colors were available for us this time.
