@@ -18,9 +18,6 @@ module OrdList (
 
 import Outputable
 
-#if __GLASGOW_HASKELL__ < 709
-import Data.Monoid ( Monoid(..) )
-#endif
 #if __GLASGOW_HASKELL__ > 710
 import Data.Semigroup   ( Semigroup )
 import qualified Data.Semigroup as Semigroup
@@ -52,6 +49,15 @@ instance Monoid (OrdList a) where
   mappend = appOL
   mconcat = concatOL
 
+instance Functor OrdList where
+  fmap = mapOL
+
+instance Foldable OrdList where
+  foldr = foldrOL
+
+instance Traversable OrdList where
+  traverse f xs = toOL <$> traverse f (fromOL xs)
+
 nilOL    :: OrdList a
 isNilOL  :: OrdList a -> Bool
 
@@ -101,9 +107,6 @@ mapOL f (Snoc xs x) = Snoc (mapOL f xs) (f x)
 mapOL f (Two x y) = Two (mapOL f x) (mapOL f y)
 mapOL f (Many xs) = Many (map f xs)
 
-instance Functor OrdList where
-  fmap = mapOL
-
 foldrOL :: (a->b->b) -> b -> OrdList a -> b
 foldrOL _ z None        = z
 foldrOL k z (One x)     = k x z
