@@ -116,7 +116,8 @@ buildPADict vect_tc prepr_ax pdata_tc pdatas_tc repr
            let body = mkLams (tvs ++ args) expr
            raw_var  <- newExportedVar (method_name dfun_name name) (exprType body)
            let var  = raw_var
-                      `setIdUnfolding` mkInlineUnfolding (Just (length args)) body
+                      `setIdUnfolding` mkInlineUnfoldingWithArity
+                                         (length args) body
                       `setInlinePragma` alwaysInlinePragma
            hoistBinding var body
            return var
