@@ -1,4 +1,7 @@
+{-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE NondecreasingIndentation #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -15,10 +18,11 @@ module Distribution.Simple.Program.Ar (
     multiStageProgramInvocation
   ) where
 
-import Control.Monad (unless)
+import Prelude ()
+import Distribution.Compat.Prelude
+
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as BS8
-import Data.Char (isSpace)
 import Distribution.Compat.CopyFile (filesEqual)
 import Distribution.Simple.LocalBuildInfo (LocalBuildInfo(..))
 import Distribution.Simple.Program
@@ -27,7 +31,7 @@ import Distribution.Simple.Program.Run
          ( programInvocation, multiStageProgramInvocation
          , runProgramInvocation )
 import Distribution.Simple.Utils
-         ( dieWithLocation, withTempDirectory )
+         ( dieWithLocation', withTempDirectory )
 import Distribution.System
          ( Arch(..), OS(..), Platform(..) )
 import Distribution.Verbosity
@@ -43,7 +47,7 @@ import System.IO
 createArLibArchive :: Verbosity -> LocalBuildInfo
                    -> FilePath -> [FilePath] -> IO ()
 createArLibArchive verbosity lbi targetPath files = do
-  (ar, _) <- requireProgram verbosity arProgram progConf
+  (ar, _) <- requireProgram verbosity arProgram progDb
 
   let (targetDir, targetName) = splitFileName targetPath
   withTempDirectory verbosity targetDir "objs" $ \ tmpDir -> do
@@ -86,12 +90,12 @@ createArLibArchive verbosity lbi targetPath files = do
 
   unless (hostArch == Arm -- See #1537
           || hostOS == AIX) $ -- AIX uses its own "ar" format variant
-    wipeMetadata tmpPath
+    wipeMetadata verbosity tmpPath
   equal <- filesEqual tmpPath targetPath
   unless equal $ renameFile tmpPath targetPath
 
   where
-    progConf = withPrograms lbi
+    progDb = withPrograms lbi
     Platform hostArch hostOS = hostPlatform lbi
     verbosityOpts v | v >= deafening = ["-v"]
                     | v >= verbose   = []
@@ -103,15 +107,15 @@ createArLibArchive verbosity lbi targetPath files = do
 -- (@-D@) flag that always writes zero for the mtime, UID and GID, and 0644
 -- for the file mode. However detecting whether @-D@ is supported seems
 -- rather harder than just re-implementing this feature.
-wipeMetadata :: FilePath -> IO ()
-wipeMetadata path = do
+wipeMetadata :: Verbosity -> FilePath -> IO ()
+wipeMetadata verbosity path = do
     -- Check for existence first (ReadWriteMode would create one otherwise)
     exists <- doesFileExist path
     unless exists $ wipeError "Temporary file disappeared"
     withBinaryFile path ReadWriteMode $ \ h -> hFileSize h >>= wipeArchive h
 
   where
-    wipeError msg = dieWithLocation path Nothing $
+    wipeError msg = dieWithLocation' verbosity path Nothing $
         "Distribution.Simple.Program.Ar.wipeMetadata: " ++ msg
     archLF = "!<arch>\x0a" -- global magic, 8 bytes
     x60LF = "\x60\x0a" -- header magic, 2 bytes
