@@ -1,3 +1,6 @@
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE RankNTypes #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Distribution.Simple.Program.Run
@@ -23,16 +26,16 @@ module Distribution.Simple.Program.Run (
     getEffectiveEnvironment,
   ) where
 
+import Prelude ()
+import Distribution.Compat.Prelude
+
 import Distribution.Simple.Program.Types
 import Distribution.Simple.Utils
 import Distribution.Verbosity
 import Distribution.Compat.Environment
 
-import Data.List
-         ( foldl', unfoldr )
 import qualified Data.Map as Map
-import Control.Monad
-         ( when )
+import System.FilePath
 import System.Exit
          ( ExitCode(..), exitWith )
 
@@ -47,6 +50,8 @@ data ProgramInvocation = ProgramInvocation {
        progInvokePath  :: FilePath,
        progInvokeArgs  :: [String],
        progInvokeEnv   :: [(String, Maybe String)],
+       -- Extra paths to add to PATH
+       progInvokePathEnv :: [FilePath],
        progInvokeCwd   :: Maybe FilePath,
        progInvokeInput :: Maybe String,
        progInvokeInputEncoding  :: IOEncoding,
@@ -62,6 +67,7 @@ emptyProgramInvocation =
     progInvokePath  = "",
     progInvokeArgs  = [],
     progInvokeEnv   = [],
+    progInvokePathEnv = [],
     progInvokeCwd   = Nothing,
     progInvokeInput = Nothing,
     progInvokeInputEncoding  = IOEncodingText,
@@ -92,6 +98,7 @@ runProgramInvocation verbosity
     progInvokePath  = path,
     progInvokeArgs  = args,
     progInvokeEnv   = [],
+    progInvokePathEnv = [],
     progInvokeCwd   = Nothing,
     progInvokeInput = Nothing
   } =
@@ -102,10 +109,12 @@ runProgramInvocation verbosity
     progInvokePath  = path,
     progInvokeArgs  = args,
     progInvokeEnv   = envOverrides,
+    progInvokePathEnv = extraPath,
     progInvokeCwd   = mcwd,
     progInvokeInput = Nothing
   } = do
-    menv <- getEffectiveEnvironment envOverrides
+    pathOverride <- getExtraPathEnv envOverrides extraPath
+    menv <- getEffectiveEnvironment (envOverrides ++ pathOverride)
     exitCode <- rawSystemIOWithEnv verbosity
                                    path args
                                    mcwd menv
@@ -118,17 +127,19 @@ runProgramInvocation verbosity
     progInvokePath  = path,
     progInvokeArgs  = args,
     progInvokeEnv   = envOverrides,
+    progInvokePathEnv = extraPath,
     progInvokeCwd   = mcwd,
     progInvokeInput = Just inputStr,
     progInvokeInputEncoding = encoding
   } = do
-    menv <- getEffectiveEnvironment envOverrides
+    pathOverride <- getExtraPathEnv envOverrides extraPath
+    menv <- getEffectiveEnvironment (envOverrides ++ pathOverride)
     (_, errors, exitCode) <- rawSystemStdInOut verbosity
                                     path args
                                     mcwd menv
                                     (Just input) True
     when (exitCode /= ExitSuccess) $
-      die $ "'" ++ path ++ "' exited with an error:\n" ++ errors
+      die' verbosity $ "'" ++ path ++ "' exited with an error:\n" ++ errors
   where
     input = case encoding of
               IOEncodingText -> (inputStr, False)
@@ -142,6 +153,7 @@ getProgramInvocationOutput verbosity
     progInvokePath  = path,
     progInvokeArgs  = args,
     progInvokeEnv   = envOverrides,
+    progInvokePathEnv = extraPath,
     progInvokeCwd   = mcwd,
     progInvokeInput = minputStr,
     progInvokeOutputEncoding = encoding
@@ -149,13 +161,14 @@ getProgramInvocationOutput verbosity
     let utf8 = case encoding of IOEncodingUTF8 -> True; _ -> False
         decode | utf8      = fromUTF8 . normaliseLineEndings
                | otherwise = id
-    menv <- getEffectiveEnvironment envOverrides
+    pathOverride <- getExtraPathEnv envOverrides extraPath
+    menv <- getEffectiveEnvironment (envOverrides ++ pathOverride)
     (output, errors, exitCode) <- rawSystemStdInOut verbosity
                                     path args
                                     mcwd menv
                                     input utf8
     when (exitCode /= ExitSuccess) $
-      die $ "'" ++ path ++ "' exited with an error:\n" ++ errors
+      die' verbosity $ "'" ++ path ++ "' exited with an error:\n" ++ errors
     return (decode output)
   where
     input =
@@ -167,10 +180,24 @@ getProgramInvocationOutput verbosity
             IOEncodingUTF8 -> (toUTF8 inputStr, True) -- use binary mode for utf8
 
 
+getExtraPathEnv :: [(String, Maybe String)] -> [FilePath] -> NoCallStackIO [(String, Maybe String)]
+getExtraPathEnv _ [] = return []
+getExtraPathEnv env extras = do
+    mb_path <- case lookup "PATH" env of
+                Just x  -> return x
+                Nothing -> lookupEnv "PATH"
+    let extra = intercalate [searchPathSeparator] extras
+        path' = case mb_path of
+                    Nothing   -> extra
+                    Just path -> extra ++ searchPathSeparator : path
+    return [("PATH", Just path')]
+
 -- | Return the current environment extended with the given overrides.
+-- If an entry is specified twice in @overrides@, the second entry takes
+-- precedence.
 --
 getEffectiveEnvironment :: [(String, Maybe String)]
-                        -> IO (Maybe [(String, String)])
+                        -> NoCallStackIO (Maybe [(String, String)])
 getEffectiveEnvironment []        = return Nothing
 getEffectiveEnvironment overrides =
     fmap (Just . Map.toList . apply overrides . Map.fromList) getEnvironment
