@@ -1,3 +1,6 @@
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE RankNTypes #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Distribution.Simple.Register
@@ -27,28 +30,43 @@ module Distribution.Simple.Register (
     register,
     unregister,
 
+    internalPackageDBPath,
+
     initPackageDB,
     doesPackageDBExist,
     createPackageDB,
     deletePackageDB,
 
+    abiHash,
     invokeHcPkg,
     registerPackage,
+    HcPkg.RegisterOptions(..),
+    HcPkg.defaultRegisterOptions,
     generateRegistrationInfo,
     inplaceInstalledPackageInfo,
     absoluteInstalledPackageInfo,
     generalInstalledPackageInfo,
   ) where
 
+import Prelude ()
+import Distribution.Compat.Prelude
+
+import Distribution.Types.TargetInfo
+import Distribution.Types.LocalBuildInfo
+import Distribution.Types.ComponentLocalBuildInfo
+
 import Distribution.Simple.LocalBuildInfo
 import Distribution.Simple.BuildPaths
+import Distribution.Simple.BuildTarget
 
 import qualified Distribution.Simple.GHC   as GHC
 import qualified Distribution.Simple.GHCJS as GHCJS
 import qualified Distribution.Simple.LHC   as LHC
 import qualified Distribution.Simple.UHC   as UHC
 import qualified Distribution.Simple.HaskellSuite as HaskellSuite
+import qualified Distribution.Simple.PackageIndex as Index
 
+import Distribution.Backpack.DescribeUnitId
 import Distribution.Simple.Compiler
 import Distribution.Simple.Program
 import Distribution.Simple.Program.Script
@@ -59,21 +77,18 @@ import Distribution.Package
 import qualified Distribution.InstalledPackageInfo as IPI
 import Distribution.InstalledPackageInfo (InstalledPackageInfo)
 import Distribution.Simple.Utils
+import Distribution.Utils.MapAccum
 import Distribution.System
 import Distribution.Text
+import Distribution.Types.ComponentName
 import Distribution.Verbosity as Verbosity
+import Distribution.Version
+import Distribution.Compat.Graph (IsNode(nodeKey))
 
 import System.FilePath ((</>), (<.>), isAbsolute)
 import System.Directory
-         ( getCurrentDirectory, removeDirectoryRecursive, removeFile
-         , doesDirectoryExist, doesFileExist )
 
-import Data.Version
-import Control.Monad (when)
-import Data.Maybe
-         ( isJust, fromMaybe, maybeToList )
-import Data.List
-         ( partition, nub )
+import Data.List (partition)
 import qualified Data.ByteString.Lazy.Char8 as BS.Char8
 
 -- -----------------------------------------------------------------------------
@@ -82,34 +97,47 @@ import qualified Data.ByteString.Lazy.Char8 as BS.Char8
 register :: PackageDescription -> LocalBuildInfo
          -> RegisterFlags -- ^Install in the user's database?; verbose
          -> IO ()
-register pkg@PackageDescription { library       = Just lib  } lbi regFlags
-  = do
-    let clbi = getComponentLocalBuildInfo lbi CLibName
+register pkg_descr lbi0 flags =
+   -- Duncan originally asked for us to not register/install files
+   -- when there was no public library.  But with per-component
+   -- configure, we legitimately need to install internal libraries
+   -- so that we can get them.  So just unconditionally install.
+   doRegister
+ where
+  doRegister = do
+    targets <- readTargetInfos verbosity pkg_descr lbi0 (regArgs flags)
 
+    -- It's important to register in build order, because ghc-pkg
+    -- will complain if a dependency is not registered.
+    let componentsToRegister
+            = neededTargetsInBuildOrder' pkg_descr lbi0 (map nodeKey targets)
+
+    (_, ipi_mbs) <-
+        mapAccumM `flip` installedPkgs lbi0 `flip` componentsToRegister $ \index tgt ->
+            case targetComponent tgt of
+                CLib lib -> do
+                    let clbi = targetCLBI tgt
+                        lbi = lbi0 { installedPkgs = index }
+                    ipi <- generateOne pkg_descr lib lbi clbi flags
+                    return (Index.insert ipi index, Just ipi)
+                _   -> return (index, Nothing)
+
+    registerAll pkg_descr lbi0 flags (catMaybes ipi_mbs)
+   where
+    verbosity = fromFlag (regVerbosity flags)
+
+generateOne :: PackageDescription -> Library -> LocalBuildInfo -> ComponentLocalBuildInfo
+            -> RegisterFlags
+            -> IO InstalledPackageInfo
+generateOne pkg lib lbi clbi regFlags
+  = do
     absPackageDBs    <- absolutePackageDBPaths packageDbs
     installedPkgInfo <- generateRegistrationInfo
                            verbosity pkg lib lbi clbi inplace reloc distPref
                            (registrationPackageDB absPackageDBs)
-
-    when (fromFlag (regPrintId regFlags)) $ do
-      putStrLn (display (IPI.installedUnitId installedPkgInfo))
-
-     -- Three different modes:
-    case () of
-     _ | modeGenerateRegFile   -> writeRegistrationFile installedPkgInfo
-       | modeGenerateRegScript -> writeRegisterScript   installedPkgInfo
-       | otherwise             -> do
-           setupMessage verbosity "Registering" (packageId pkg)
-           registerPackage verbosity (compiler lbi) (withPrograms lbi) False
-                           packageDbs installedPkgInfo
-
+    info verbosity (IPI.showInstalledPackageInfo installedPkgInfo)
+    return installedPkgInfo
   where
-    modeGenerateRegFile = isJust (flagToMaybe (regGenPkgConf regFlags))
-    regFile             = fromMaybe (display (packageId pkg) <.> "conf")
-                                    (fromFlag (regGenPkgConf regFlags))
-
-    modeGenerateRegScript = fromFlag (regGenScript regFlags)
-
     inplace   = fromFlag (regInPlace regFlags)
     reloc     = relocatable lbi
     -- FIXME: there's really no guarantee this will work.
@@ -120,22 +148,70 @@ register pkg@PackageDescription { library       = Just lib  } lbi regFlags
     distPref  = fromFlag (regDistPref regFlags)
     verbosity = fromFlag (regVerbosity regFlags)
 
-    writeRegistrationFile installedPkgInfo = do
-      notice verbosity ("Creating package registration file: " ++ regFile)
-      writeUTF8File regFile (IPI.showInstalledPackageInfo installedPkgInfo)
+registerAll :: PackageDescription -> LocalBuildInfo -> RegisterFlags
+            -> [InstalledPackageInfo]
+            -> IO ()
+registerAll pkg lbi regFlags ipis
+  = do
+    when (fromFlag (regPrintId regFlags)) $ do
+      for_ ipis $ \installedPkgInfo ->
+        -- Only print the public library's IPI
+        when (packageId installedPkgInfo == packageId pkg
+              && IPI.sourceLibName installedPkgInfo == Nothing) $
+          putStrLn (display (IPI.installedUnitId installedPkgInfo))
 
-    writeRegisterScript installedPkgInfo =
+     -- Three different modes:
+    case () of
+     _ | modeGenerateRegFile   -> writeRegistrationFileOrDirectory
+       | modeGenerateRegScript -> writeRegisterScript
+       | otherwise             -> do
+           for_ ipis $ \ipi -> do
+               setupMessage' verbosity "Registering" (packageId pkg)
+                 (libraryComponentName (IPI.sourceLibName ipi))
+                 (Just (IPI.instantiatedWith ipi))
+               registerPackage verbosity (compiler lbi) (withPrograms lbi)
+                               packageDbs ipi HcPkg.defaultRegisterOptions
+
+  where
+    modeGenerateRegFile = isJust (flagToMaybe (regGenPkgConf regFlags))
+    regFile             = fromMaybe (display (packageId pkg) <.> "conf")
+                                    (fromFlag (regGenPkgConf regFlags))
+
+    modeGenerateRegScript = fromFlag (regGenScript regFlags)
+
+    -- FIXME: there's really no guarantee this will work.
+    -- registering into a totally different db stack can
+    -- fail if dependencies cannot be satisfied.
+    packageDbs = nub $ withPackageDB lbi
+                    ++ maybeToList (flagToMaybe  (regPackageDB regFlags))
+    verbosity = fromFlag (regVerbosity regFlags)
+
+    writeRegistrationFileOrDirectory = do
+      -- Handles overwriting both directory and file
+      deletePackageDB regFile
+      case ipis of
+        [installedPkgInfo] -> do
+          info verbosity ("Creating package registration file: " ++ regFile)
+          writeUTF8File regFile (IPI.showInstalledPackageInfo installedPkgInfo)
+        _ -> do
+          info verbosity ("Creating package registration directory: " ++ regFile)
+          createDirectory regFile
+          let num_ipis = length ipis
+              lpad m xs = replicate (m - length ys) '0' ++ ys
+                  where ys = take m xs
+              number i = lpad (length (show num_ipis)) (show i)
+          for_ (zip ([1..] :: [Int]) ipis) $ \(i, installedPkgInfo) ->
+            writeUTF8File (regFile </> (number i ++ "-" ++ display (IPI.installedUnitId installedPkgInfo)))
+                          (IPI.showInstalledPackageInfo installedPkgInfo)
+
+    writeRegisterScript =
       case compilerFlavor (compiler lbi) of
         JHC -> notice verbosity "Registration scripts not needed for jhc"
         UHC -> notice verbosity "Registration scripts not needed for uhc"
-        _   -> withHcPkg
+        _   -> withHcPkg verbosity
                "Registration scripts are not implemented for this compiler"
                (compiler lbi) (withPrograms lbi)
-               (writeHcPkgRegisterScript verbosity installedPkgInfo packageDbs)
-
-register _ _ regFlags = notice verbosity "No package to register"
-  where
-    verbosity = fromFlag (regVerbosity regFlags)
+               (writeHcPkgRegisterScript verbosity ipis packageDbs)
 
 
 generateRegistrationInfo :: Verbosity
@@ -152,29 +228,45 @@ generateRegistrationInfo verbosity pkg lib lbi clbi inplace reloc distPref packa
   --TODO: eliminate pwd!
   pwd <- getCurrentDirectory
 
-  --TODO: the method of setting the UnitId is compiler specific
-  --      this aspect should be delegated to a per-compiler helper.
-  let comp = compiler lbi
-  abi_hash <-
-    case compilerFlavor comp of
-     GHC | compilerVersion comp >= Version [6,11] [] -> do
-            fmap AbiHash $ GHC.libAbiHash verbosity pkg lbi lib clbi
-     GHCJS -> do
-            fmap AbiHash $ GHCJS.libAbiHash verbosity pkg lbi lib clbi
-     _ -> return (AbiHash "")
-
   installedPkgInfo <-
     if inplace
+      -- NB: With an inplace installation, the user may run './Setup
+      -- build' to update the library files, without reregistering.
+      -- In this case, it is critical that the ABI hash not flip.
       then return (inplaceInstalledPackageInfo pwd distPref
-                     pkg abi_hash lib lbi clbi)
-    else if reloc
-      then relocRegistrationInfo verbosity
-                     pkg lib lbi clbi abi_hash packageDb
-      else return (absoluteInstalledPackageInfo
-                     pkg abi_hash lib lbi clbi)
+                     pkg (mkAbiHash "inplace") lib lbi clbi)
+    else do
+        abi_hash <- abiHash verbosity pkg distPref lbi lib clbi
+        if reloc
+          then relocRegistrationInfo verbosity
+                         pkg lib lbi clbi abi_hash packageDb
+          else return (absoluteInstalledPackageInfo
+                         pkg abi_hash lib lbi clbi)
 
 
-  return installedPkgInfo{ IPI.abiHash = abi_hash }
+  return installedPkgInfo
+
+-- | Compute the 'AbiHash' of a library that we built inplace.
+abiHash :: Verbosity
+        -> PackageDescription
+        -> FilePath
+        -> LocalBuildInfo
+        -> Library
+        -> ComponentLocalBuildInfo
+        -> IO AbiHash
+abiHash verbosity pkg distPref lbi lib clbi =
+    case compilerFlavor comp of
+     GHC | compilerVersion comp >= mkVersion [6,11] -> do
+            fmap mkAbiHash $ GHC.libAbiHash verbosity pkg lbi' lib clbi
+     GHCJS -> do
+            fmap mkAbiHash $ GHCJS.libAbiHash verbosity pkg lbi' lib clbi
+     _ -> return (mkAbiHash "")
+  where
+    comp = compiler lbi
+    lbi' = lbi {
+              withPackageDB = withPackageDB lbi
+                  ++ [SpecificPackageDB (internalPackageDBPath lbi distPref)]
+           }
 
 relocRegistrationInfo :: Verbosity
                       -> PackageDescription
@@ -189,15 +281,16 @@ relocRegistrationInfo verbosity pkg lib lbi clbi abi_hash packageDb =
     GHC -> do fs <- GHC.pkgRoot verbosity lbi packageDb
               return (relocatableInstalledPackageInfo
                         pkg abi_hash lib lbi clbi fs)
-    _   -> die "Distribution.Simple.Register.relocRegistrationInfo: \
+    _   -> die' verbosity
+              "Distribution.Simple.Register.relocRegistrationInfo: \
                \not implemented for this compiler"
 
-initPackageDB :: Verbosity -> Compiler -> ProgramConfiguration -> FilePath -> IO ()
+initPackageDB :: Verbosity -> Compiler -> ProgramDb -> FilePath -> IO ()
 initPackageDB verbosity comp progdb dbPath =
     createPackageDB verbosity comp progdb False dbPath
 
 -- | Create an empty package DB at the specified location.
-createPackageDB :: Verbosity -> Compiler -> ProgramConfiguration -> Bool
+createPackageDB :: Verbosity -> Compiler -> ProgramDb -> Bool
                 -> FilePath -> IO ()
 createPackageDB verbosity comp progdb preferCompat dbPath =
     case compilerFlavor comp of
@@ -206,10 +299,11 @@ createPackageDB verbosity comp progdb preferCompat dbPath =
       LHC   -> HcPkg.init (LHC.hcPkgInfo   progdb) verbosity False dbPath
       UHC   -> return ()
       HaskellSuite _ -> HaskellSuite.initPackageDB verbosity progdb dbPath
-      _              -> die $ "Distribution.Simple.Register.createPackageDB: "
+      _              -> die' verbosity $
+                              "Distribution.Simple.Register.createPackageDB: "
                            ++ "not implemented for this compiler"
 
-doesPackageDBExist :: FilePath -> IO Bool
+doesPackageDBExist :: FilePath -> NoCallStackIO Bool
 doesPackageDBExist dbPath = do
     -- currently one impl for all compiler flavours, but could change if needed
     dir_exists <- doesDirectoryExist dbPath
@@ -217,7 +311,7 @@ doesPackageDBExist dbPath = do
         then return True
         else doesFileExist dbPath
 
-deletePackageDB :: FilePath -> IO ()
+deletePackageDB :: FilePath -> NoCallStackIO ()
 deletePackageDB dbPath = do
     -- currently one impl for all compiler flavours, but could change if needed
     dir_exists <- doesDirectoryExist dbPath
@@ -228,53 +322,58 @@ deletePackageDB dbPath = do
 
 -- | Run @hc-pkg@ using a given package DB stack, directly forwarding the
 -- provided command-line arguments to it.
-invokeHcPkg :: Verbosity -> Compiler -> ProgramConfiguration -> PackageDBStack
+invokeHcPkg :: Verbosity -> Compiler -> ProgramDb -> PackageDBStack
                 -> [String] -> IO ()
-invokeHcPkg verbosity comp conf dbStack extraArgs =
-  withHcPkg "invokeHcPkg" comp conf
+invokeHcPkg verbosity comp progdb dbStack extraArgs =
+  withHcPkg verbosity "invokeHcPkg" comp progdb
     (\hpi -> HcPkg.invoke hpi verbosity dbStack extraArgs)
 
-withHcPkg :: String -> Compiler -> ProgramConfiguration
+withHcPkg :: Verbosity -> String -> Compiler -> ProgramDb
           -> (HcPkg.HcPkgInfo -> IO a) -> IO a
-withHcPkg name comp conf f =
+withHcPkg verbosity name comp progdb f =
   case compilerFlavor comp of
-    GHC   -> f (GHC.hcPkgInfo conf)
-    GHCJS -> f (GHCJS.hcPkgInfo conf)
-    LHC   -> f (LHC.hcPkgInfo conf)
-    _     -> die ("Distribution.Simple.Register." ++ name ++ ":\
+    GHC   -> f (GHC.hcPkgInfo progdb)
+    GHCJS -> f (GHCJS.hcPkgInfo progdb)
+    LHC   -> f (LHC.hcPkgInfo progdb)
+    _     -> die' verbosity ("Distribution.Simple.Register." ++ name ++ ":\
                   \not implemented for this compiler")
 
 registerPackage :: Verbosity
                 -> Compiler
-                -> ProgramConfiguration
-                -> Bool
+                -> ProgramDb
                 -> PackageDBStack
                 -> InstalledPackageInfo
+                -> HcPkg.RegisterOptions
                 -> IO ()
-registerPackage verbosity comp progdb multiInstance packageDbs installedPkgInfo =
+registerPackage verbosity comp progdb packageDbs installedPkgInfo registerOptions =
   case compilerFlavor comp of
-    GHC   -> GHC.registerPackage   verbosity progdb multiInstance packageDbs installedPkgInfo
-    GHCJS -> GHCJS.registerPackage verbosity progdb multiInstance packageDbs installedPkgInfo
-    _ | multiInstance
-          -> die "Registering multiple package instances is not yet supported for this compiler"
-    LHC   -> LHC.registerPackage   verbosity      progdb packageDbs installedPkgInfo
+    GHC   -> GHC.registerPackage   verbosity progdb packageDbs installedPkgInfo registerOptions
+    GHCJS -> GHCJS.registerPackage verbosity progdb packageDbs installedPkgInfo registerOptions
+    _ | HcPkg.registerMultiInstance registerOptions
+          -> die' verbosity "Registering multiple package instances is not yet supported for this compiler"
+    LHC   -> LHC.registerPackage   verbosity      progdb packageDbs installedPkgInfo registerOptions
     UHC   -> UHC.registerPackage   verbosity comp progdb packageDbs installedPkgInfo
     JHC   -> notice verbosity "Registering for jhc (nothing to do)"
     HaskellSuite {} ->
       HaskellSuite.registerPackage verbosity      progdb packageDbs installedPkgInfo
-    _    -> die "Registering is not implemented for this compiler"
+    _    -> die' verbosity "Registering is not implemented for this compiler"
 
 writeHcPkgRegisterScript :: Verbosity
-                         -> InstalledPackageInfo
+                         -> [InstalledPackageInfo]
                          -> PackageDBStack
                          -> HcPkg.HcPkgInfo
                          -> IO ()
-writeHcPkgRegisterScript verbosity installedPkgInfo packageDbs hpi = do
-  let invocation  = HcPkg.reregisterInvocation hpi Verbosity.normal
-                      packageDbs (Right installedPkgInfo)
-      regScript   = invocationAsSystemScript buildOS invocation
+writeHcPkgRegisterScript verbosity ipis packageDbs hpi = do
+  let genScript installedPkgInfo =
+          let invocation  = HcPkg.registerInvocation hpi Verbosity.normal
+                              packageDbs installedPkgInfo
+                              HcPkg.defaultRegisterOptions
+          in invocationAsSystemScript buildOS invocation
+      scripts = map genScript ipis
+      -- TODO: Do something more robust here
+      regScript = unlines scripts
 
-  notice verbosity ("Creating package registration script: " ++ regScriptFileName)
+  info verbosity ("Creating package registration script: " ++ regScriptFileName)
   writeUTF8File regScriptFileName regScript
   setFileExecutable regScriptFileName
 
@@ -302,8 +401,11 @@ generalInstalledPackageInfo
   -> InstalledPackageInfo
 generalInstalledPackageInfo adjustRelIncDirs pkg abi_hash lib lbi clbi installDirs =
   IPI.InstalledPackageInfo {
-    IPI.sourcePackageId    = packageId   pkg,
+    IPI.sourcePackageId    = packageId pkg,
     IPI.installedUnitId    = componentUnitId clbi,
+    IPI.installedComponentId_ = componentComponentId clbi,
+    IPI.instantiatedWith   = componentInstantiatedWith clbi,
+    IPI.sourceLibName      = libName lib,
     IPI.compatPackageKey   = componentCompatPackageKey clbi,
     IPI.license            = license     pkg,
     IPI.copyright          = copyright   pkg,
@@ -316,13 +418,12 @@ generalInstalledPackageInfo adjustRelIncDirs pkg abi_hash lib lbi clbi installDi
     IPI.description        = description pkg,
     IPI.category           = category    pkg,
     IPI.abiHash            = abi_hash,
+    IPI.indefinite         = componentIsIndefinite clbi,
     IPI.exposed            = libExposed  lib,
     IPI.exposedModules     = componentExposedModules clbi,
     IPI.hiddenModules      = otherModules bi,
     IPI.trusted            = IPI.trusted IPI.emptyInstalledPackageInfo,
     IPI.importDirs         = [ libdir installDirs | hasModules ],
-    -- Note. the libsubdir and datasubdir templates have already been expanded
-    -- into libdir and datadir.
     IPI.libraryDirs        = libdirs,
     IPI.libraryDynDirs     = dynlibdirs,
     IPI.dataDir            = datadir installDirs,
@@ -333,7 +434,8 @@ generalInstalledPackageInfo adjustRelIncDirs pkg abi_hash lib lbi clbi installDi
     IPI.extraGHCiLibraries = extraGHCiLibs bi,
     IPI.includeDirs        = absinc ++ adjustRelIncDirs relinc,
     IPI.includes           = includes bi,
-    IPI.depends            = map fst (componentPackageDeps clbi),
+    IPI.depends            = depends,
+    IPI.abiDepends         = abi_depends,
     IPI.ccOptions          = [], -- Note. NOT ccOptions bi!
                                  -- We don't want cc-options to be propagated
                                  -- to C compilations in other packages.
@@ -346,12 +448,23 @@ generalInstalledPackageInfo adjustRelIncDirs pkg abi_hash lib lbi clbi installDi
   }
   where
     bi = libBuildInfo lib
+    --TODO: unclear what the root cause of the
+    -- duplication is, but we nub it here for now:
+    depends = ordNub $ map fst (componentPackageDeps clbi)
+    abi_depends = map add_abi depends
+    add_abi uid = IPI.AbiDependency uid abi
+      where
+        abi = case Index.lookupUnitId (installedPkgs lbi) uid of
+                Nothing -> error $
+                  "generalInstalledPackageInfo: missing IPI for " ++ display uid
+                Just ipi -> IPI.abiHash ipi
     (absinc, relinc) = partition isAbsolute (includeDirs bi)
-    hasModules = not $ null (libModules lib)
+    hasModules = not $ null (allLibModules lib clbi)
     comp = compiler lbi
-    hasLibrary = hasModules || not (null (cSources bi))
-                            || (not (null (jsSources bi)) &&
-                                compilerFlavor comp == GHCJS)
+    hasLibrary = (hasModules || not (null (cSources bi))
+                             || (not (null (jsSources bi)) &&
+                                compilerFlavor comp == GHCJS))
+               && not (componentIsIndefinite clbi)
     (libdirs, dynlibdirs)
       | not hasLibrary
       = (extraLibDirs bi, [])
@@ -385,11 +498,9 @@ inplaceInstalledPackageInfo inplaceDir distPref pkg abi_hash lib lbi clbi =
                                 pkg abi_hash lib lbi clbi installDirs
   where
     adjustRelativeIncludeDirs = map (inplaceDir </>)
-    libTargetDir
-        | componentUnitId clbi == localUnitId lbi = buildDir lbi
-        | otherwise = buildDir lbi </> display (componentUnitId clbi)
+    libTargetDir = componentBuildDir lbi clbi
     installDirs =
-      (absoluteInstallDirs pkg lbi NoCopyDest) {
+      (absoluteComponentInstallDirs pkg lbi (componentUnitId clbi) NoCopyDest) {
         libdir     = inplaceDir </> libTargetDir,
         dynlibdir  = inplaceDir </> libTargetDir,
         datadir    = inplaceDir </> dataDir pkg,
@@ -422,7 +533,7 @@ absoluteInstalledPackageInfo pkg abi_hash lib lbi clbi =
       | null (installIncludes bi) = []
       | otherwise                 = [includedir installDirs]
     bi = libBuildInfo lib
-    installDirs = absoluteInstallDirs pkg lbi NoCopyDest
+    installDirs = absoluteComponentInstallDirs pkg lbi (componentUnitId clbi) NoCopyDest
 
 
 relocatableInstalledPackageInfo :: PackageDescription
@@ -444,7 +555,7 @@ relocatableInstalledPackageInfo pkg abi_hash lib lbi clbi pkgroot =
     bi = libBuildInfo lib
 
     installDirs = fmap (("${pkgroot}" </>) . shortRelativePath pkgroot)
-                $ absoluteInstallDirs pkg lbi NoCopyDest
+                $ absoluteComponentInstallDirs pkg lbi (componentUnitId clbi) NoCopyDest
 
 -- -----------------------------------------------------------------------------
 -- Unregistration
@@ -464,10 +575,16 @@ unregister pkg lbi regFlags = do
                     (BS.Char8.pack $ invocationAsSystemScript buildOS invocation)
              else runProgramInvocation verbosity invocation
   setupMessage verbosity "Unregistering" pkgid
-  withHcPkg "unregistering is only implemented for GHC and GHCJS"
+  withHcPkg verbosity "unregistering is only implemented for GHC and GHCJS"
     (compiler lbi) (withPrograms lbi) unreg
 
 unregScriptFileName :: FilePath
 unregScriptFileName = case buildOS of
                           Windows -> "unregister.bat"
                           _       -> "unregister.sh"
+
+internalPackageDBPath :: LocalBuildInfo -> FilePath -> FilePath
+internalPackageDBPath lbi distPref =
+      case compilerFlavor (compiler lbi) of
+        UHC -> UHC.inplacePackageDbPath lbi
+        _   -> distPref </> "package.conf.inplace"
