@@ -32,6 +32,7 @@ module Control.Exception.Base (
         BlockedIndefinitelyOnMVar(..),
         BlockedIndefinitelyOnSTM(..),
         AllocationLimitExceeded(..),
+        CompactionFailed(..),
         Deadlock(..),
         NoMethodError(..),
         PatternMatchFail(..),
@@ -110,45 +111,6 @@ import Data.Either
 -----------------------------------------------------------------------------
 -- Catching exceptions
 
--- |This is the simplest of the exception-catching functions.  It
--- takes a single argument, runs it, and if an exception is raised
--- the \"handler\" is executed, with the value of the exception passed as an
--- argument.  Otherwise, the result is returned as normal.  For example:
---
--- >   catch (readFile f)
--- >         (\e -> do let err = show (e :: IOException)
--- >                   hPutStr stderr ("Warning: Couldn't open " ++ f ++ ": " ++ err)
--- >                   return "")
---
--- Note that we have to give a type signature to @e@, or the program
--- will not typecheck as the type is ambiguous. While it is possible
--- to catch exceptions of any type, see the section \"Catching all
--- exceptions\" (in "Control.Exception") for an explanation of the problems with doing so.
---
--- For catching exceptions in pure (non-'IO') expressions, see the
--- function 'evaluate'.
---
--- Note that due to Haskell\'s unspecified evaluation order, an
--- expression may throw one of several possible exceptions: consider
--- the expression @(error \"urk\") + (1 \`div\` 0)@.  Does
--- the expression throw
--- @ErrorCall \"urk\"@, or @DivideByZero@?
---
--- The answer is \"it might throw either\"; the choice is
--- non-deterministic. If you are catching any type of exception then you
--- might catch either. If you are calling @catch@ with type
--- @IO Int -> (ArithException -> IO Int) -> IO Int@ then the handler may
--- get run with @DivideByZero@ as an argument, or an @ErrorCall \"urk\"@
--- exception may be propogated further up. If you call it again, you
--- might get a the opposite behaviour. This is ok, because 'catch' is an
--- 'IO' computation.
---
-catch   :: Exception e
-        => IO a         -- ^ The computation to run
-        -> (e -> IO a)  -- ^ Handler to invoke if an exception is raised
-        -> IO a
-catch act = catchException (lazy act)
-
 -- | The function 'catchJust' is like 'catch', but it takes an extra
 -- argument which is an /exception predicate/, a function which
 -- selects which type of exceptions we\'re interested in.
@@ -299,9 +261,11 @@ bracketOnError before after thing =
 -- source location of the pattern.
 newtype PatternMatchFail = PatternMatchFail String
 
+-- | @since 4.0
 instance Show PatternMatchFail where
     showsPrec _ (PatternMatchFail err) = showString err
 
+-- | @since 4.0
 instance Exception PatternMatchFail
 
 -----
@@ -313,9 +277,11 @@ instance Exception PatternMatchFail
 -- location of the record selector.
 newtype RecSelError = RecSelError String
 
+-- | @since 4.0
 instance Show RecSelError where
     showsPrec _ (RecSelError err) = showString err
 
+-- | @since 4.0
 instance Exception RecSelError
 
 -----
@@ -325,9 +291,11 @@ instance Exception RecSelError
 -- constructed.
 newtype RecConError = RecConError String
 
+-- | @since 4.0
 instance Show RecConError where
     showsPrec _ (RecConError err) = showString err
 
+-- | @since 4.0
 instance Exception RecConError
 
 -----
@@ -339,9 +307,11 @@ instance Exception RecConError
 -- location of the record update.
 newtype RecUpdError = RecUpdError String
 
+-- | @since 4.0
 instance Show RecUpdError where
     showsPrec _ (RecUpdError err) = showString err
 
+-- | @since 4.0
 instance Exception RecUpdError
 
 -----
@@ -351,9 +321,11 @@ instance Exception RecUpdError
 -- @String@ gives information about which method it was.
 newtype NoMethodError = NoMethodError String
 
+-- | @since 4.0
 instance Show NoMethodError where
     showsPrec _ (NoMethodError err) = showString err
 
+-- | @since 4.0
 instance Exception NoMethodError
 
 -----
@@ -365,9 +337,11 @@ instance Exception NoMethodError
 -- @since 4.9.0.0
 newtype TypeError = TypeError String
 
+-- | @since 4.9.0.0
 instance Show TypeError where
     showsPrec _ (TypeError err) = showString err
 
+-- | @since 4.9.0.0
 instance Exception TypeError
 
 -----
@@ -378,9 +352,11 @@ instance Exception TypeError
 -- guaranteed to terminate or not.
 data NonTermination = NonTermination
 
+-- | @since 4.0
 instance Show NonTermination where
     showsPrec _ NonTermination = showString "<<loop>>"
 
+-- | @since 4.0
 instance Exception NonTermination
 
 -----
@@ -389,9 +365,11 @@ instance Exception NonTermination
 -- package, inside another call to @atomically@.
 data NestedAtomically = NestedAtomically
 
+-- | @since 4.0
 instance Show NestedAtomically where
     showsPrec _ NestedAtomically = showString "Control.Concurrent.STM.atomically was nested"
 
+-- | @since 4.0
 instance Exception NestedAtomically
 
 -----
