@@ -34,6 +34,7 @@
 module Data.Type.Equality (
   -- * The equality types
   (:~:)(..), type (~~),
+  (:~~:)(..),
 
   -- * Working with equality
   sym, trans, castWith, gcastWith, apply, inner, outer,
@@ -55,7 +56,7 @@ import Data.Type.Bool
 -- | Lifted, homogeneous equality. By lifted, we mean that it can be
 -- bogus (deferred type error). By homogeneous, the two types @a@
 -- and @b@ must have the same kind.
-class a ~~ b => (a :: k) ~ (b :: k) | a -> b, b -> a
+class a ~~ b => (a :: k) ~ (b :: k)
   -- See Note [The equality types story] in TysPrim
   -- NB: All this class does is to wrap its superclass, which is
   --     the "real", inhomogeneous equality; this is needed when
@@ -63,6 +64,11 @@ class a ~~ b => (a :: k) ~ (b :: k) | a -> b, b -> a
   -- NB: Not exported, as (~) is magical syntax. That's also why there's
   -- no fixity.
 
+  -- It's tempting to put functional dependencies on (~), but it's not
+  -- necessary because the functional-dependency coverage check looks
+  -- through superclasses, and (~#) is handled in that check.
+
+-- | @since 4.9.0.0
 instance {-# INCOHERENT #-} a ~~ b => a ~ b
   -- See Note [The equality types story] in TysPrim
   -- If we have a Wanted (t1 ~ t2), we want to immediately
@@ -71,7 +77,7 @@ instance {-# INCOHERENT #-} a ~~ b => a ~ b
   -- INCOHERENT because we want to use this instance eagerly, even when
   -- the tyvars are partially unknown.
 
-infix 4 :~:
+infix 4 :~:, :~~:
 
 -- | Propositional equality. If @a :~: b@ is inhabited by some terminating
 -- value, then the type @a@ is the same as the type @b@. To use this equality
@@ -106,7 +112,7 @@ gcastWith Refl x = x
 apply :: (f :~: g) -> (a :~: b) -> (f a :~: g b)
 apply Refl Refl = Refl
 
--- | Extract equality of the arguments from an equality of a applied types
+-- | Extract equality of the arguments from an equality of applied types
 inner :: (f a :~: g b) -> (a :~: b)
 inner Refl = Refl
 
@@ -118,18 +124,47 @@ deriving instance Eq   (a :~: b)
 deriving instance Show (a :~: b)
 deriving instance Ord  (a :~: b)
 
+-- | @since 4.7.0.0
 instance a ~ b => Read (a :~: b) where
   readsPrec d = readParen (d > 10) (\r -> [(Refl, s) | ("Refl",s) <- lex r ])
 
+-- | @since 4.7.0.0
 instance a ~ b => Enum (a :~: b) where
   toEnum 0 = Refl
   toEnum _ = errorWithoutStackTrace "Data.Type.Equality.toEnum: bad argument"
 
   fromEnum Refl = 0
 
-instance a ~ b => Bounded (a :~: b) where
-  minBound = Refl
-  maxBound = Refl
+-- | @since 4.7.0.0
+deriving instance a ~ b => Bounded (a :~: b)
+
+-- | Kind heterogeneous propositional equality. Like '(:~:)', @a :~~: b@ is
+-- inhabited by a terminating value if and only if @a@ is the same type as @b@.
+--
+-- @since 4.10.0.0
+data (a :: k1) :~~: (b :: k2) where
+   HRefl :: a :~~: a
+
+-- | @since 4.10.0.0
+deriving instance Eq   (a :~~: b)
+-- | @since 4.10.0.0
+deriving instance Show (a :~~: b)
+-- | @since 4.10.0.0
+deriving instance Ord  (a :~~: b)
+
+-- | @since 4.10.0.0
+instance a ~~ b => Read (a :~~: b) where
+  readsPrec d = readParen (d > 10) (\r -> [(HRefl, s) | ("HRefl",s) <- lex r ])
+
+-- | @since 4.10.0.0
+instance a ~~ b => Enum (a :~~: b) where
+  toEnum 0 = HRefl
+  toEnum _ = errorWithoutStackTrace "Data.Type.Equality.toEnum: bad argument"
+
+  fromEnum HRefl = 0
+
+-- | @since 4.10.0.0
+deriving instance a ~~ b => Bounded (a :~~: b)
 
 -- | This class contains types where you can learn the equality of two types
 -- from information contained in /terms/. Typically, only singleton types should
@@ -138,9 +173,14 @@ class TestEquality f where
   -- | Conditionally prove the equality of @a@ and @b@.
   testEquality :: f a -> f b -> Maybe (a :~: b)
 
+-- | @since 4.7.0.0
 instance TestEquality ((:~:) a) where
   testEquality Refl Refl = Just Refl
 
+-- | @since 4.10.0.0
+instance TestEquality ((:~~:) a) where
+  testEquality HRefl HRefl = Just Refl
+
 -- | A type family to compute Boolean equality. Instances are provided
 -- only for /open/ kinds, such as @*@ and function kinds. Instances are
 -- also provided for datatypes exported from base. A poly-kinded instance
