@@ -147,6 +147,31 @@ readParen b g   =  if b then mandatory else optional
 -- >                 up_prec = 5
 -- >
 -- >         readListPrec = readListPrecDefault
+--
+-- Why do both 'readsPrec' and 'readPrec' exist, and why does GHC opt to
+-- implement 'readPrec' in derived 'Read' instances instead of 'readsPrec'?
+-- The reason is that 'readsPrec' is based on the 'ReadS' type, and although
+-- 'ReadS' is mentioned in the Haskell 2010 Report, it is not a very efficient
+-- parser data structure.
+--
+-- 'readPrec', on the other hand, is based on a much more efficient 'ReadPrec'
+-- datatype (a.k.a \"new-style parsers\"), but its definition relies on the use
+-- of the @RankNTypes@ language extension. Therefore, 'readPrec' (and its
+-- cousin, 'readListPrec') are marked as GHC-only. Nevertheless, it is
+-- recommended to use 'readPrec' instead of 'readsPrec' whenever possible
+-- for the efficiency improvements it brings.
+--
+-- As mentioned above, derived 'Read' instances in GHC will implement
+-- 'readPrec' instead of 'readsPrec'. The default implementations of
+-- 'readsPrec' (and its cousin, 'readList') will simply use 'readPrec' under
+-- the hood. If you are writing a 'Read' instance by hand, it is recommended
+-- to write it like so:
+--
+-- @
+-- instance 'Read' T where
+--   'readPrec'     = ...
+--   'readListPrec' = 'readListPrecDefault'
+-- @
 
 class Read a where
   {-# MINIMAL readsPrec | readPrec #-}
@@ -270,17 +295,6 @@ expectCharP c a = do
     else pfail
 {-# INLINE expectCharP #-}
 
--- A version of skipSpaces that takes the next
--- parser as an argument. That is,
---
--- skipSpacesThenP m = lift skipSpaces >> m
---
--- Since skipSpaces is recursive, it appears that we get
--- cleaner code by providing the continuation explicitly.
--- In particular, we avoid passing an extra continuation
--- of the form
---
--- \ () -> ...
 skipSpacesThenP :: ReadPrec a -> ReadPrec a
 skipSpacesThenP m =
   do s <- look
@@ -294,14 +308,6 @@ paren :: ReadPrec a -> ReadPrec a
 --      where @p@ parses \"P0\" in precedence context zero
 paren p = skipSpacesThenP (paren' p)
 
--- We try very hard to make paren' efficient, because parens is ubiquitous.
--- Earlier code used `expectP` to look for the parentheses. The problem is that
--- this lexes a (potentially long) token just to check if it's a parenthesis or
--- not. So the first token of pretty much every value would be fully lexed
--- twice. Now, we look for the '(' by hand instead. Since there's no reason not
--- to, and it allows for faster failure, we do the same for ')'. This strategy
--- works particularly well here because neither '(' nor ')' can begin any other
--- lexeme.
 paren' :: ReadPrec a -> ReadPrec a
 paren' p = expectCharP '(' $ reset p >>= \x ->
               skipSpacesThenP (expectCharP ')' (pure x))
@@ -311,9 +317,9 @@ parens :: ReadPrec a -> ReadPrec a
 --      where @p@ parses \"P\"  in the current precedence context
 --          and parses \"P0\" in precedence context zero
 parens p = optional
- where
-  optional  = p +++ mandatory
-  mandatory = paren optional
+  where
+    optional = skipSpacesThenP (p +++ mandatory)
+    mandatory = paren' optional
 
 list :: ReadPrec a -> ReadPrec [a]
 -- ^ @(list p)@ parses a list of things parsed by @p@,
@@ -356,6 +362,7 @@ choose sps = foldr ((+++) . try_one) pfail sps
 
 deriving instance Read GeneralCategory
 
+-- | @since 2.01
 instance Read Char where
   readPrec =
     parens
@@ -373,6 +380,7 @@ instance Read Char where
 
   readList = readListDefault
 
+-- | @since 2.01
 instance Read Bool where
   readPrec =
     parens
@@ -386,6 +394,7 @@ instance Read Bool where
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance Read Ordering where
   readPrec =
     parens
@@ -427,6 +436,7 @@ parenthesis-like objects such as (...) and [...] can be an argument to
 'Just'.
 -}
 
+-- | @since 2.01
 instance Read a => Read (Maybe a) where
   readPrec =
     parens
@@ -442,6 +452,7 @@ instance Read a => Read (Maybe a) where
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance Read a => Read [a] where
   {-# SPECIALISE instance Read [String] #-}
   {-# SPECIALISE instance Read [Char] #-}
@@ -450,6 +461,7 @@ instance Read a => Read [a] where
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance  (Ix a, Read a, Read b) => Read (Array a b)  where
     readPrec = parens $ prec appPrec $
                do expectP (L.Ident "array")
@@ -460,6 +472,7 @@ instance  (Ix a, Read a, Read b) => Read (Array a b)  where
     readListPrec = readListPrecDefault
     readList     = readListDefault
 
+-- | @since 2.01
 instance Read L.Lexeme where
   readPrec     = lexP
   readListPrec = readListPrecDefault
@@ -497,29 +510,35 @@ convertFrac (L.Number n) = let resRange = floatRange (undefined :: a)
                               Just rat -> return $ fromRational rat
 convertFrac _            = pfail
 
+-- | @since 2.01
 instance Read Int where
   readPrec     = readNumber convertInt
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 4.5.0.0
 instance Read Word where
     readsPrec p s = [(fromInteger x, r) | (x, r) <- readsPrec p s]
 
+-- | @since 2.01
 instance Read Integer where
   readPrec     = readNumber convertInt
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance Read Float where
   readPrec     = readNumber convertFrac
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance Read Double where
   readPrec     = readNumber convertFrac
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Integral a, Read a) => Read (Ratio a) where
   readPrec =
     parens
@@ -539,6 +558,7 @@ instance (Integral a, Read a) => Read (Ratio a) where
 -- Tuple instances of Read, up to size 15
 ------------------------------------------------------------------------
 
+-- | @since 2.01
 instance Read () where
   readPrec =
     parens
@@ -550,6 +570,7 @@ instance Read () where
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b) => Read (a,b) where
   readPrec = wrap_tup read_tup2
   readListPrec = readListPrecDefault
@@ -583,6 +604,7 @@ read_tup8 = do  (a,b,c,d) <- read_tup4
                 return (a,b,c,d,e,f,g,h)
 
 
+-- | @since 2.01
 instance (Read a, Read b, Read c) => Read (a, b, c) where
   readPrec = wrap_tup (do { (a,b) <- read_tup2; read_comma
                           ; c <- readPrec
@@ -590,11 +612,13 @@ instance (Read a, Read b, Read c) => Read (a, b, c) where
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d) => Read (a, b, c, d) where
   readPrec = wrap_tup read_tup4
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e) => Read (a, b, c, d, e) where
   readPrec = wrap_tup (do { (a,b,c,d) <- read_tup4; read_comma
                           ; e <- readPrec
@@ -602,6 +626,7 @@ instance (Read a, Read b, Read c, Read d, Read e) => Read (a, b, c, d, e) where
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f)
         => Read (a, b, c, d, e, f) where
   readPrec = wrap_tup (do { (a,b,c,d) <- read_tup4; read_comma
@@ -610,6 +635,7 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f)
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g)
         => Read (a, b, c, d, e, f, g) where
   readPrec = wrap_tup (do { (a,b,c,d) <- read_tup4; read_comma
@@ -619,12 +645,14 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g)
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h)
         => Read (a, b, c, d, e, f, g, h) where
   readPrec     = wrap_tup read_tup8
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
           Read i)
         => Read (a, b, c, d, e, f, g, h, i) where
@@ -634,6 +662,7 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
           Read i, Read j)
         => Read (a, b, c, d, e, f, g, h, i, j) where
@@ -643,6 +672,7 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
           Read i, Read j, Read k)
         => Read (a, b, c, d, e, f, g, h, i, j, k) where
@@ -653,6 +683,7 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
           Read i, Read j, Read k, Read l)
         => Read (a, b, c, d, e, f, g, h, i, j, k, l) where
@@ -662,6 +693,7 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
           Read i, Read j, Read k, Read l, Read m)
         => Read (a, b, c, d, e, f, g, h, i, j, k, l, m) where
@@ -672,6 +704,7 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
           Read i, Read j, Read k, Read l, Read m, Read n)
         => Read (a, b, c, d, e, f, g, h, i, j, k, l, m, n) where
@@ -682,6 +715,7 @@ instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
   readListPrec = readListPrecDefault
   readList     = readListDefault
 
+-- | @since 2.01
 instance (Read a, Read b, Read c, Read d, Read e, Read f, Read g, Read h,
           Read i, Read j, Read k, Read l, Read m, Read n, Read o)
         => Read (a, b, c, d, e, f, g, h, i, j, k, l, m, n, o) where
