@@ -12,7 +12,6 @@
 --                (c) Joachim Breitner 2011
 -- License     :  BSD-style
 -- Maintainer  :  libraries@haskell.org
--- Stability   :  provisional
 -- Portability :  portable
 --
 -- An efficient implementation of integer sets.
@@ -140,7 +139,7 @@ module Data.IntSet (
 #endif
             ) where
 
-import Data.IntSet.Base as IS
+import Data.IntSet.Internal as IS
 
 -- $strictness
 --
