@@ -26,6 +26,7 @@
 #include "sm/GC.h" // for gcWorkerThread()
 #include "STM.h"
 #include "RtsUtils.h"
+#include "sm/OSMem.h"
 
 #if !defined(mingw32_HOST_OS)
 #include "rts/IOManager.h" // for setIOManagerControlFd()
@@ -37,8 +38,8 @@
 // builds, and for +RTS -N1
 Capability MainCapability;
 
-nat n_capabilities = 0;
-nat enabled_capabilities = 0;
+uint32_t n_capabilities = 0;
+uint32_t enabled_capabilities = 0;
 
 // The array of Capabilities.  It's important that when we need
 // to allocate more Capabilities we don't have to move the existing
@@ -51,7 +52,7 @@ Capability **capabilities = NULL;
 // an in-call has a chance of quickly finding a free Capability.
 // Maintaining a global free list of Capabilities would require global
 // locking, so we don't do that.
-Capability *last_free_capability = NULL;
+static Capability *last_free_capability[MAX_NUMA_NODES];
 
 /*
  * Indicates that the RTS wants to synchronise all the Capabilities
@@ -59,6 +60,12 @@ Capability *last_free_capability = NULL;
  */
 PendingSync * volatile pending_sync = 0;
 
+// Number of logical NUMA nodes
+uint32_t n_numa_nodes;
+
+// Map logical NUMA node to OS node numbers
+uint32_t numa_map[MAX_NUMA_NODES];
+
 /* Let foreign code get the current Capability -- assuming there is one!
  * This is useful for unsafe foreign calls because they are called with
  * the current Capability held, but they are not passed it. For example,
@@ -75,7 +82,7 @@ Capability * rts_unsafeGetMyCapability (void)
 }
 
 #if defined(THREADED_RTS)
-STATIC_INLINE rtsBool
+STATIC_INLINE bool
 globalWorkToDo (void)
 {
     return sched_state >= SCHED_INTERRUPTING
@@ -89,8 +96,8 @@ findSpark (Capability *cap)
 {
   Capability *robbed;
   StgClosurePtr spark;
-  rtsBool retry;
-  nat i = 0;
+  bool retry;
+  uint32_t i = 0;
 
   if (!emptyRunQueue(cap) || cap->n_returning_tasks != 0) {
       // If there are other threads, don't try to run any new
@@ -100,7 +107,7 @@ findSpark (Capability *cap)
   }
 
   do {
-      retry = rtsFalse;
+      retry = false;
 
       // first try to get a spark from our own pool.
       // We should be using reclaimSpark(), because it works without
@@ -123,7 +130,7 @@ findSpark (Capability *cap)
           return spark;
       }
       if (!emptySparkPoolCap(cap)) {
-          retry = rtsTrue;
+          retry = true;
       }
 
       if (n_capabilities == 1) { return NULL; } // makes no sense...
@@ -151,7 +158,7 @@ findSpark (Capability *cap)
           if (spark == NULL && !emptySparkPoolCap(robbed)) {
               // we conflicted with another thread while trying to steal;
               // try again later.
-              retry = rtsTrue;
+              retry = true;
           }
 
           if (spark != NULL) {
@@ -172,17 +179,17 @@ findSpark (Capability *cap)
 // The result is only valid for an instant, of course, so in a sense
 // is immediately invalid, and should not be relied upon for
 // correctness.
-rtsBool
+bool
 anySparks (void)
 {
-    nat i;
+    uint32_t i;
 
     for (i=0; i < n_capabilities; i++) {
         if (!emptySparkPoolCap(capabilities[i])) {
-            return rtsTrue;
+            return true;
         }
     }
-    return rtsFalse;
+    return false;
 }
 #endif
 
@@ -234,14 +241,15 @@ popReturningTask (Capability *cap)
  * ------------------------------------------------------------------------- */
 
 static void
-initCapability( Capability *cap, nat i )
+initCapability (Capability *cap, uint32_t i)
 {
-    nat g;
+    uint32_t g;
 
     cap->no = i;
-    cap->in_haskell        = rtsFalse;
+    cap->node = capNoToNumaNode(i);
+    cap->in_haskell        = false;
     cap->idle              = 0;
-    cap->disabled          = rtsFalse;
+    cap->disabled          = false;
 
     cap->run_queue_hd      = END_TSO_QUEUE;
     cap->run_queue_tl      = END_TSO_QUEUE;
@@ -258,6 +266,7 @@ initCapability( Capability *cap, nat i )
     cap->returning_tasks_tl = NULL;
     cap->n_returning_tasks  = 0;
     cap->inbox              = (Message*)END_TSO_QUEUE;
+    cap->putMVars           = NULL;
     cap->sparks             = allocSparkPool();
     cap->spark_stats.created    = 0;
     cap->spark_stats.dud        = 0;
@@ -323,33 +332,60 @@ initCapability( Capability *cap, nat i )
  *            controlled by the user via the RTS flag -N.
  *
  * ------------------------------------------------------------------------- */
-void
-initCapabilities( void )
+void initCapabilities (void)
 {
+    uint32_t i;
+
     /* Declare a couple capability sets representing the process and
        clock domain. Each capability will get added to these capsets. */
     traceCapsetCreate(CAPSET_OSPROCESS_DEFAULT, CapsetTypeOsProcess);
     traceCapsetCreate(CAPSET_CLOCKDOMAIN_DEFAULT, CapsetTypeClockdomain);
 
+    // Initialise NUMA
+    if (!RtsFlags.GcFlags.numa) {
+        n_numa_nodes = 1;
+        for (i = 0; i < MAX_NUMA_NODES; i++) {
+            numa_map[i] = 0;
+        }
+    } else {
+        uint32_t nNodes = osNumaNodes();
+        if (nNodes > MAX_NUMA_NODES) {
+            barf("Too many NUMA nodes (max %d)", MAX_NUMA_NODES);
+        }
+        StgWord mask = RtsFlags.GcFlags.numaMask & osNumaMask();
+        uint32_t logical = 0, physical = 0;
+        for (; physical < MAX_NUMA_NODES; physical++) {
+            if (mask & 1) {
+                numa_map[logical++] = physical;
+            }
+            mask = mask >> 1;
+        }
+        n_numa_nodes = logical;
+        if (logical == 0) {
+            barf("%s: available NUMA node set is empty");
+        }
+    }
+
 #if defined(THREADED_RTS)
 
 #ifndef REG_Base
     // We can't support multiple CPUs if BaseReg is not a register
-    if (RtsFlags.ParFlags.nNodes > 1) {
+    if (RtsFlags.ParFlags.nCapabilities > 1) {
         errorBelch("warning: multiple CPUs not supported in this build, reverting to 1");
-        RtsFlags.ParFlags.nNodes = 1;
+        RtsFlags.ParFlags.nCapabilities = 1;
     }
 #endif
 
     n_capabilities = 0;
-    moreCapabilities(0, RtsFlags.ParFlags.nNodes);
-    n_capabilities = RtsFlags.ParFlags.nNodes;
+    moreCapabilities(0, RtsFlags.ParFlags.nCapabilities);
+    n_capabilities = RtsFlags.ParFlags.nCapabilities;
 
 #else /* !THREADED_RTS */
 
     n_capabilities = 1;
     capabilities = stgMallocBytes(sizeof(Capability*), "initCapabilities");
     capabilities[0] = &MainCapability;
+
     initCapability(&MainCapability, 0);
 
 #endif
@@ -359,14 +395,16 @@ initCapabilities( void )
     // There are no free capabilities to begin with.  We will start
     // a worker Task to each Capability, which will quickly put the
     // Capability on the free list when it finds nothing to do.
-    last_free_capability = capabilities[0];
+    for (i = 0; i < n_numa_nodes; i++) {
+        last_free_capability[i] = capabilities[0];
+    }
 }
 
 void
-moreCapabilities (nat from USED_IF_THREADS, nat to USED_IF_THREADS)
+moreCapabilities (uint32_t from USED_IF_THREADS, uint32_t to USED_IF_THREADS)
 {
 #if defined(THREADED_RTS)
-    nat i;
+    uint32_t i;
     Capability **old_capabilities = capabilities;
 
     capabilities = stgMallocBytes(to * sizeof(Capability*), "moreCapabilities");
@@ -406,7 +444,7 @@ moreCapabilities (nat from USED_IF_THREADS, nat to USED_IF_THREADS)
 
 void contextSwitchAllCapabilities(void)
 {
-    nat i;
+    uint32_t i;
     for (i=0; i < n_capabilities; i++) {
         contextSwitchCapability(capabilities[i]);
     }
@@ -414,7 +452,7 @@ void contextSwitchAllCapabilities(void)
 
 void interruptAllCapabilities(void)
 {
-    nat i;
+    uint32_t i;
     for (i=0; i < n_capabilities; i++) {
         interruptCapability(capabilities[i]);
     }
@@ -435,7 +473,7 @@ void interruptAllCapabilities(void)
  * ------------------------------------------------------------------------- */
 
 #if defined(THREADED_RTS)
-STATIC_INLINE void
+static void
 giveCapabilityToTask (Capability *cap USED_IF_DEBUG, Task *task)
 {
     ASSERT_LOCK_HELD(&cap->lock);
@@ -444,8 +482,8 @@ giveCapabilityToTask (Capability *cap USED_IF_DEBUG, Task *task)
                cap->no, task->incall->tso ? "bound task" : "worker",
                serialisableTaskId(task));
     ACQUIRE_LOCK(&task->lock);
-    if (task->wakeup == rtsFalse) {
-        task->wakeup = rtsTrue;
+    if (task->wakeup == false) {
+        task->wakeup = true;
         // the wakeup flag is needed because signalCondition() doesn't
         // flag the condition if the thread is already runniing, but we want
         // it to be sticky.
@@ -465,7 +503,7 @@ giveCapabilityToTask (Capability *cap USED_IF_DEBUG, Task *task)
 #if defined(THREADED_RTS)
 void
 releaseCapability_ (Capability* cap,
-                    rtsBool always_wakeup)
+                    bool always_wakeup)
 {
     Task *task;
 
@@ -540,7 +578,7 @@ releaseCapability_ (Capability* cap,
 #ifdef PROFILING
     cap->r.rCCCS = CCS_IDLE;
 #endif
-    last_free_capability = cap;
+    last_free_capability[cap->node] = cap;
     debugTrace(DEBUG_sched, "freeing capability %d", cap->no);
 }
 
@@ -548,7 +586,7 @@ void
 releaseCapability (Capability* cap USED_IF_THREADS)
 {
     ACQUIRE_LOCK(&cap->lock);
-    releaseCapability_(cap, rtsFalse);
+    releaseCapability_(cap, false);
     RELEASE_LOCK(&cap->lock);
 }
 
@@ -556,7 +594,7 @@ void
 releaseAndWakeupCapability (Capability* cap USED_IF_THREADS)
 {
     ACQUIRE_LOCK(&cap->lock);
-    releaseCapability_(cap, rtsTrue);
+    releaseCapability_(cap, true);
     RELEASE_LOCK(&cap->lock);
 }
 
@@ -582,7 +620,7 @@ enqueueWorker (Capability* cap USED_IF_THREADS)
     {
         debugTrace(DEBUG_sched, "%d spare workers already, exiting",
                    cap->n_spare_workers);
-        releaseCapability_(cap,rtsFalse);
+        releaseCapability_(cap,false);
         // hold the lock until after workerTaskStop; c.f. scheduleWorker()
         workerTaskStop(task);
         RELEASE_LOCK(&cap->lock);
@@ -610,7 +648,7 @@ static Capability * waitForWorkerCapability (Task *task)
         // task->lock held, cap->lock not held
         if (!task->wakeup) waitCondition(&task->cond, &task->lock);
         cap = task->cap;
-        task->wakeup = rtsFalse;
+        task->wakeup = false;
         RELEASE_LOCK(&task->lock);
 
         debugTrace(DEBUG_sched, "woken up on capability %d", cap->no);
@@ -675,7 +713,7 @@ static Capability * waitForReturnCapability (Task *task)
         // task->lock held, cap->lock not held
         if (!task->wakeup) waitCondition(&task->cond, &task->lock);
         cap = task->cap;
-        task->wakeup = rtsFalse;
+        task->wakeup = false;
         RELEASE_LOCK(&task->lock);
 
         // now check whether we should wake up...
@@ -719,24 +757,32 @@ void waitForCapability (Capability **pCap, Task *task)
     *pCap = &MainCapability;
 
 #else
+    uint32_t i;
     Capability *cap = *pCap;
 
     if (cap == NULL) {
-        // Try last_free_capability first
-        cap = last_free_capability;
-        if (cap->running_task) {
-            nat i;
-            // otherwise, search for a free capability
-            cap = NULL;
-            for (i = 0; i < n_capabilities; i++) {
-                if (!capabilities[i]->running_task) {
-                    cap = capabilities[i];
-                    break;
+        if (task->preferred_capability != -1) {
+            cap = capabilities[task->preferred_capability %
+                               enabled_capabilities];
+        } else {
+            // Try last_free_capability first
+            cap = last_free_capability[task->node];
+            if (cap->running_task) {
+                // Otherwise, search for a free capability on this node.
+                cap = NULL;
+                for (i = task->node; i < enabled_capabilities;
+                     i += n_numa_nodes) {
+                    // visits all the capabilities on this node, because
+                    // cap[i]->node == i % n_numa_nodes
+                    if (!capabilities[i]->running_task) {
+                        cap = capabilities[i];
+                        break;
+                    }
+                }
+                if (cap == NULL) {
+                    // Can't find a free one, use last_free_capability.
+                    cap = last_free_capability[task->node];
                 }
-            }
-            if (cap == NULL) {
-                // Can't find a free one, use last_free_capability.
-                cap = last_free_capability;
             }
         }
 
@@ -797,9 +843,9 @@ void waitForCapability (Capability **pCap, Task *task)
 #if defined (THREADED_RTS)
 
 /* See Note [GC livelock] in Schedule.c for why we have gcAllowed
-   and return the rtsBool */
-rtsBool /* Did we GC? */
-yieldCapability (Capability** pCap, Task *task, rtsBool gcAllowed)
+   and return the bool */
+bool /* Did we GC? */
+yieldCapability (Capability** pCap, Task *task, bool gcAllowed)
 {
     Capability *cap = *pCap;
 
@@ -815,7 +861,7 @@ yieldCapability (Capability** pCap, Task *task, rtsBool gcAllowed)
                 traceSparkCounters(cap);
                 // See Note [migrated bound threads 2]
                 if (task->cap == cap) {
-                    return rtsTrue;
+                    return true;
                 }
             }
         }
@@ -824,7 +870,7 @@ yieldCapability (Capability** pCap, Task *task, rtsBool gcAllowed)
     debugTrace(DEBUG_sched, "giving up capability %d", cap->no);
 
     // We must now release the capability and wait to be woken up again.
-    task->wakeup = rtsFalse;
+    task->wakeup = false;
 
     ACQUIRE_LOCK(&cap->lock);
 
@@ -833,7 +879,7 @@ yieldCapability (Capability** pCap, Task *task, rtsBool gcAllowed)
         enqueueWorker(cap);
     }
 
-    releaseCapability_(cap, rtsFalse);
+    releaseCapability_(cap, false);
 
     if (isWorker(task) || isBoundTask(task)) {
         RELEASE_LOCK(&cap->lock);
@@ -860,7 +906,7 @@ yieldCapability (Capability** pCap, Task *task, rtsBool gcAllowed)
 
     ASSERT_FULL_CAPABILITY_INVARIANTS(cap,task);
 
-    return rtsFalse;
+    return false;
 }
 
 #endif /* THREADED_RTS */
@@ -908,7 +954,7 @@ prodCapability (Capability *cap, Task *task)
     ACQUIRE_LOCK(&cap->lock);
     if (!cap->running_task) {
         cap->running_task = task;
-        releaseCapability_(cap,rtsTrue);
+        releaseCapability_(cap,true);
     }
     RELEASE_LOCK(&cap->lock);
 }
@@ -924,19 +970,21 @@ prodCapability (Capability *cap, Task *task)
 
 #if defined (THREADED_RTS)
 
-rtsBool
+bool
 tryGrabCapability (Capability *cap, Task *task)
 {
-    if (cap->running_task != NULL) return rtsFalse;
-    ACQUIRE_LOCK(&cap->lock);
+    int r;
+    if (cap->running_task != NULL) return false;
+    r = TRY_ACQUIRE_LOCK(&cap->lock);
+    if (r != 0) return false;
     if (cap->running_task != NULL) {
         RELEASE_LOCK(&cap->lock);
-        return rtsFalse;
+        return false;
     }
     task->cap = cap;
     cap->running_task = task;
     RELEASE_LOCK(&cap->lock);
-    return rtsTrue;
+    return true;
 }
 
 
@@ -957,13 +1005,13 @@ tryGrabCapability (Capability *cap, Task *task)
  *
  * ------------------------------------------------------------------------- */
 
-void
+static void
 shutdownCapability (Capability *cap USED_IF_THREADS,
                     Task *task USED_IF_THREADS,
-                    rtsBool safe USED_IF_THREADS)
+                    bool safe USED_IF_THREADS)
 {
 #if defined(THREADED_RTS)
-    nat i;
+    uint32_t i;
 
     task->cap = cap;
 
@@ -1014,7 +1062,7 @@ shutdownCapability (Capability *cap USED_IF_THREADS,
         if (!emptyRunQueue(cap) || cap->spare_workers) {
             debugTrace(DEBUG_sched,
                        "runnable threads or workers still alive, yielding");
-            releaseCapability_(cap,rtsFalse); // this will wake up a worker
+            releaseCapability_(cap,false); // this will wake up a worker
             RELEASE_LOCK(&cap->lock);
             yieldThread();
             continue;
@@ -1058,9 +1106,9 @@ shutdownCapability (Capability *cap USED_IF_THREADS,
 }
 
 void
-shutdownCapabilities(Task *task, rtsBool safe)
+shutdownCapabilities(Task *task, bool safe)
 {
-    nat i;
+    uint32_t i;
     for (i=0; i < n_capabilities; i++) {
         ASSERT(task->incall->tso == NULL);
         shutdownCapability(capabilities[i], task, safe);
@@ -1087,7 +1135,7 @@ void
 freeCapabilities (void)
 {
 #if defined(THREADED_RTS)
-    nat i;
+    uint32_t i;
     for (i=0; i < n_capabilities; i++) {
         freeCapability(capabilities[i]);
         if (capabilities[i] != &MainCapability)
@@ -1109,7 +1157,7 @@ freeCapabilities (void)
 
 void
 markCapability (evac_fn evac, void *user, Capability *cap,
-                rtsBool no_mark_sparks USED_IF_THREADS)
+                bool no_mark_sparks USED_IF_THREADS)
 {
     InCall *incall;
 
@@ -1141,18 +1189,18 @@ markCapability (evac_fn evac, void *user, Capability *cap,
 void
 markCapabilities (evac_fn evac, void *user)
 {
-    nat n;
+    uint32_t n;
     for (n = 0; n < n_capabilities; n++) {
-        markCapability(evac, user, capabilities[n], rtsFalse);
+        markCapability(evac, user, capabilities[n], false);
     }
 }
 
 #if defined(THREADED_RTS)
-rtsBool checkSparkCountInvariant (void)
+bool checkSparkCountInvariant (void)
 {
     SparkCounters sparks = { 0, 0, 0, 0, 0, 0 };
     StgWord64 remaining = 0;
-    nat i;
+    uint32_t i;
 
     for (i = 0; i < n_capabilities; i++) {
         sparks.created   += capabilities[i]->spark_stats.created;
@@ -1179,7 +1227,8 @@ rtsBool checkSparkCountInvariant (void)
 #endif
 
 #if !defined(mingw32_HOST_OS)
-void setIOManagerControlFd(nat cap_no USED_IF_THREADS, int fd USED_IF_THREADS) {
+void
+setIOManagerControlFd(uint32_t cap_no USED_IF_THREADS, int fd USED_IF_THREADS) {
 #if defined(THREADED_RTS)
     if (cap_no < n_capabilities) {
         capabilities[cap_no]->io_manager_control_wr_fd = fd;
