@@ -38,7 +38,7 @@
 // object as referenced so that it won't get unloaded in this round.
 //
 
-static void checkAddress (HashTable *addrs, void *addr)
+static void checkAddress (HashTable *addrs, const void *addr)
 {
     ObjectCode *oc;
     int i;
@@ -73,7 +73,7 @@ static void searchStackChunk (HashTable *addrs, StgPtr sp, StgPtr stack_end)
         switch (info->i.type) {
         case RET_SMALL:
         case RET_BIG:
-            checkAddress(addrs, (void*)info);
+            checkAddress(addrs, (const void*)info);
             break;
 
         default:
@@ -88,9 +88,9 @@ static void searchStackChunk (HashTable *addrs, StgPtr sp, StgPtr stack_end)
 static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
 {
     StgPtr p;
-    StgInfoTable *info;
-    nat size;
-    rtsBool prim;
+    const StgInfoTable *info;
+    uint32_t size;
+    bool prim;
 
     for (; bd != NULL; bd = bd->link) {
 
@@ -102,7 +102,7 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
         p = bd->start;
         while (p < bd->free) {
             info = get_itbl((StgClosure *)p);
-            prim = rtsFalse;
+            prim = false;
 
             switch (info->type) {
 
@@ -122,13 +122,14 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
                 size = sizeofW(StgThunkHeader) + 1;
                 break;
 
-            case CONSTR:
             case FUN:
             case FUN_1_0:
             case FUN_0_1:
             case FUN_1_1:
             case FUN_0_2:
             case FUN_2_0:
+            case CONSTR:
+            case CONSTR_NOCAF:
             case CONSTR_1_0:
             case CONSTR_0_1:
             case CONSTR_1_1:
@@ -137,10 +138,9 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
                 size = sizeW_fromITBL(info);
                 break;
 
-            case IND_PERM:
             case BLACKHOLE:
             case BLOCKING_QUEUE:
-                prim = rtsTrue;
+                prim = true;
                 size = sizeW_fromITBL(info);
                 break;
 
@@ -152,12 +152,12 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
                 // blackholes when it calls raiseAsync() on the
                 // resurrected threads.  So we know that any IND will
                 // be the size of a BLACKHOLE.
-                prim = rtsTrue;
+                prim = true;
                 size = BLACKHOLE_sizeW();
                 break;
 
             case BCO:
-                prim = rtsTrue;
+                prim = true;
                 size = bco_sizeW((StgBCO *)p);
                 break;
 
@@ -169,24 +169,24 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
             case MUT_PRIM:
             case MUT_VAR_CLEAN:
             case MUT_VAR_DIRTY:
-                prim = rtsTrue;
+                prim = true;
                 size = sizeW_fromITBL(info);
                 break;
 
             case AP:
-                prim = rtsTrue;
+                prim = true;
                 size = ap_sizeW((StgAP *)p);
                 break;
 
             case PAP:
-                prim = rtsTrue;
+                prim = true;
                 size = pap_sizeW((StgPAP *)p);
                 break;
 
             case AP_STACK:
             {
                 StgAP_STACK *ap = (StgAP_STACK *)p;
-                prim = rtsTrue;
+                prim = true;
                 size = ap_stack_sizeW(ap);
                 searchStackChunk(addrs, (StgPtr)ap->payload,
                                  (StgPtr)ap->payload + ap->size);
@@ -194,7 +194,7 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
             }
 
             case ARR_WORDS:
-                prim = rtsTrue;
+                prim = true;
                 size = arr_words_sizeW((StgArrBytes*)p);
                 break;
 
@@ -202,7 +202,7 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
             case MUT_ARR_PTRS_DIRTY:
             case MUT_ARR_PTRS_FROZEN:
             case MUT_ARR_PTRS_FROZEN0:
-                prim = rtsTrue;
+                prim = true;
                 size = mut_arr_ptrs_sizeW((StgMutArrPtrs *)p);
                 break;
 
@@ -210,18 +210,18 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
             case SMALL_MUT_ARR_PTRS_DIRTY:
             case SMALL_MUT_ARR_PTRS_FROZEN:
             case SMALL_MUT_ARR_PTRS_FROZEN0:
-                prim = rtsTrue;
+                prim = true;
                 size = small_mut_arr_ptrs_sizeW((StgSmallMutArrPtrs *)p);
                 break;
 
             case TSO:
-                prim = rtsTrue;
+                prim = true;
                 size = sizeofW(StgTSO);
                 break;
 
             case STACK: {
                 StgStack *stack = (StgStack*)p;
-                prim = rtsTrue;
+                prim = true;
                 searchStackChunk(addrs, stack->sp,
                                  stack->stack + stack->stack_size);
                 size = stack_sizeW(stack);
@@ -229,7 +229,7 @@ static void searchHeapBlocks (HashTable *addrs, bdescr *bd)
             }
 
             case TREC_CHUNK:
-                prim = rtsTrue;
+                prim = true;
                 size = sizeofW(StgTRecChunk);
                 break;
 
@@ -276,7 +276,7 @@ static void searchCostCentres (HashTable *addrs, CostCentreStack *ccs)
 //
 void checkUnload (StgClosure *static_objects)
 {
-  nat g, n;
+  uint32_t g, n;
   HashTable *addrs;
   StgClosure* p;
   const StgInfoTable *info;
@@ -292,7 +292,7 @@ void checkUnload (StgClosure *static_objects)
   for (oc = unloaded_objects; oc; oc = oc->next) {
       IF_DEBUG(linker, debugBelch("Checking whether to unload %" PATH_FMT "\n",
                                   oc->fileName));
-      oc->referenced = rtsFalse;
+      oc->referenced = false;
   }
 
   addrs = allocHashTable();
