@@ -14,24 +14,23 @@
 extern void        printPtr        ( StgPtr p );
 extern void        printObj        ( StgClosure *obj );
 
-extern char *      closure_type_names[];
+extern const char *  closure_type_names[];
 
-void               info_hdr_type   ( StgClosure *closure, char *res );
-char  *            info_type       ( StgClosure *closure );
-char  *            info_type_by_ip ( StgInfoTable *ip );
-char  *            info_update_frame ( StgClosure *closure );
+void               info_hdr_type   ( const StgClosure *closure, char *res );
+const char  *      info_type       ( const StgClosure *closure );
+const char  *      info_type_by_ip ( const StgInfoTable *ip );
+const char  *      info_update_frame ( const StgClosure *closure );
 
 #ifdef DEBUG
-extern void        prettyPrintClosure (StgClosure *obj);
-extern void        printClosure    ( StgClosure *obj );
+extern void        printClosure    ( const StgClosure *obj );
 extern void        printStackChunk ( StgPtr sp, StgPtr spLim );
 extern void        printTSO        ( StgTSO *tso );
 
-extern void DEBUG_LoadSymbols( char *name );
+extern void DEBUG_LoadSymbols( const char *name );
 
 extern const char *lookupGHCName( void *addr );
 
-extern char *what_next_strs[];
+extern const char *what_next_strs[];
 #endif
 
 #include "EndPrivate.h"
