@@ -9,6 +9,7 @@
 #include "PosixSource.h"
 #include "Rts.h"
 
+#include "Capability.h"
 #include "RtsFlags.h"
 #include "RtsUtils.h"
 #include "Profiling.h"
@@ -19,6 +20,7 @@
 #include "LdvProfile.h"
 #include "Arena.h"
 #include "Printer.h"
+#include "Trace.h"
 #include "sm/GCThread.h"
 
 #include <string.h>
@@ -37,7 +39,7 @@
  * store only up to (max_era - 1) as its creation or last use time.
  * -------------------------------------------------------------------------- */
 unsigned int era;
-static nat max_era;
+static uint32_t max_era;
 
 /* -----------------------------------------------------------------------------
  * Counters
@@ -48,15 +50,16 @@ static nat max_era;
  * lag/drag/void counters for each identity.
  * -------------------------------------------------------------------------- */
 typedef struct _counter {
-    void *identity;
+    const void *identity;
     union {
-        nat resid;
+        ssize_t resid;
         struct {
-            long prim;     // total size of 'inherently used' closures
-            long not_used; // total size of 'never used' closures
-            long used;     // total size of 'used at least once' closures
-            long void_total;  // current total size of 'destroyed without being used' closures
-            long drag_total;  // current total size of 'used at least once and waiting to die'
+            // Total sizes of:
+            ssize_t prim;     // 'inherently used' closures
+            ssize_t not_used; // 'never used' closures
+            ssize_t used;     // 'used at least once' closures
+            ssize_t void_total;  // 'destroyed without being used' closures
+            ssize_t drag_total;  // 'used at least once and waiting to die'
         } ldv;
     } c;
     struct _counter *next;
@@ -79,15 +82,15 @@ typedef struct {
     Arena     * arena;
 
     // for LDV profiling, when just displaying by LDV
-    long       prim;
-    long       not_used;
-    long       used;
-    long       void_total;
-    long       drag_total;
+    ssize_t    prim;
+    ssize_t    not_used;
+    ssize_t    used;
+    ssize_t    void_total;
+    ssize_t    drag_total;
 } Census;
 
 static Census *censuses = NULL;
-static nat n_censuses = 0;
+static uint32_t n_censuses = 0;
 
 #ifdef PROFILING
 static void aggregateCensusInfo( void );
@@ -95,15 +98,15 @@ static void aggregateCensusInfo( void );
 
 static void dumpCensus( Census *census );
 
-static rtsBool closureSatisfiesConstraints( StgClosure* p );
+static bool closureSatisfiesConstraints( const StgClosure* p );
 
 /* ----------------------------------------------------------------------------
  * Find the "closure identity", which is a unique pointer representing
  * the band to which this closure's heap space is attributed in the
  * heap profile.
  * ------------------------------------------------------------------------- */
-static void *
-closureIdentity( StgClosure *p )
+static const void *
+closureIdentity( const StgClosure *p )
 {
     switch (RtsFlags.ProfFlags.doHeapProfile) {
 
@@ -127,7 +130,7 @@ closureIdentity( StgClosure *p )
 #else
     case HEAP_BY_CLOSURE_TYPE:
     {
-        StgInfoTable *info;
+        const StgInfoTable *info;
         info = get_itbl(p);
         switch (info->type) {
         case CONSTR:
@@ -136,8 +139,7 @@ closureIdentity( StgClosure *p )
         case CONSTR_2_0:
         case CONSTR_1_1:
         case CONSTR_0_2:
-        case CONSTR_STATIC:
-        case CONSTR_NOCAF_STATIC:
+        case CONSTR_NOCAF:
             return GET_CON_DESC(itbl_to_con_itbl(info));
         default:
             return closure_type_names[info->type];
@@ -154,14 +156,14 @@ closureIdentity( StgClosure *p )
  * Profiling type predicates
  * ----------------------------------------------------------------------- */
 #ifdef PROFILING
-STATIC_INLINE rtsBool
+STATIC_INLINE bool
 doingLDVProfiling( void )
 {
     return (RtsFlags.ProfFlags.doHeapProfile == HEAP_BY_LDV
             || RtsFlags.ProfFlags.bioSelector != NULL);
 }
 
-rtsBool
+bool
 doingRetainerProfiling( void )
 {
     return (RtsFlags.ProfFlags.doHeapProfile == HEAP_BY_RETAINER
@@ -180,10 +182,10 @@ doingRetainerProfiling( void )
 
 #ifdef PROFILING
 void
-LDV_recordDead( StgClosure *c, nat size )
+LDV_recordDead( const StgClosure *c, uint32_t size )
 {
-    void *id;
-    nat t;
+    const void *id;
+    uint32_t t;
     counter *ctr;
 
     if (era > 0 && closureSatisfiesConstraints(c)) {
@@ -193,14 +195,14 @@ LDV_recordDead( StgClosure *c, nat size )
             t = (LDVW((c)) & LDV_CREATE_MASK) >> LDV_SHIFT;
             if (t < era) {
                 if (RtsFlags.ProfFlags.bioSelector == NULL) {
-                    censuses[t].void_total   += (long)size;
-                    censuses[era].void_total -= (long)size;
+                    censuses[t].void_total   += size;
+                    censuses[era].void_total -= size;
                     ASSERT(censuses[t].void_total < censuses[t].not_used);
                 } else {
                     id = closureIdentity(c);
                     ctr = lookupHashTable(censuses[t].hash, (StgWord)id);
                     ASSERT( ctr != NULL );
-                    ctr->c.ldv.void_total += (long)size;
+                    ctr->c.ldv.void_total += size;
                     ctr = lookupHashTable(censuses[era].hash, (StgWord)id);
                     if (ctr == NULL) {
                         ctr = arenaAlloc(censuses[era].arena, sizeof(counter));
@@ -210,7 +212,7 @@ LDV_recordDead( StgClosure *c, nat size )
                         ctr->next = censuses[era].ctrs;
                         censuses[era].ctrs = ctr;
                     }
-                    ctr->c.ldv.void_total -= (long)size;
+                    ctr->c.ldv.void_total -= size;
                 }
             }
         } else {
@@ -220,11 +222,11 @@ LDV_recordDead( StgClosure *c, nat size )
                     censuses[t+1].drag_total += size;
                     censuses[era].drag_total -= size;
                 } else {
-                    void *id;
+                    const void *id;
                     id = closureIdentity(c);
                     ctr = lookupHashTable(censuses[t+1].hash, (StgWord)id);
                     ASSERT( ctr != NULL );
-                    ctr->c.ldv.drag_total += (long)size;
+                    ctr->c.ldv.drag_total += size;
                     ctr = lookupHashTable(censuses[era].hash, (StgWord)id);
                     if (ctr == NULL) {
                         ctr = arenaAlloc(censuses[era].arena, sizeof(counter));
@@ -234,7 +236,7 @@ LDV_recordDead( StgClosure *c, nat size )
                         ctr->next = censuses[era].ctrs;
                         censuses[era].ctrs = ctr;
                     }
-                    ctr->c.ldv.drag_total -= (long)size;
+                    ctr->c.ldv.drag_total -= size;
                 }
             }
         }
@@ -281,7 +283,7 @@ nextEra( void )
 
         if (era == max_era) {
             errorBelch("Maximum number of censuses reached.");
-            if (rtsConfig.rts_opts_suggestions == rtsTrue) {
+            if (rtsConfig.rts_opts_suggestions == true) {
                 if (rtsConfig.rts_opts_enabled == RtsOptsAll)  {
                     errorBelch("Use `+RTS -i' to reduce censuses.");
                 } else  {
@@ -359,7 +361,7 @@ void endProfiling( void )
 #endif /* !PROFILING */
 
 static void
-printSample(rtsBool beginSample, StgDouble sampleValue)
+printSample(bool beginSample, StgDouble sampleValue)
 {
     fprintf(hp_file, "%s %f\n",
             (beginSample ? "BEGIN_SAMPLE" : "END_SAMPLE"),
@@ -369,10 +371,23 @@ printSample(rtsBool beginSample, StgDouble sampleValue)
     }
 }
 
+static void
+dumpCostCentresToEventLog(void)
+{
+#ifdef PROFILING
+    CostCentre *cc, *next;
+    for (cc = CC_LIST; cc != NULL; cc = next) {
+        next = cc->link;
+        traceHeapProfCostCentre(cc->ccID, cc->label, cc->module,
+                                cc->srcloc, cc->is_caf);
+    }
+#endif
+}
+
 /* --------------------------------------------------------------------------
  * Initialize the heap profilier
  * ----------------------------------------------------------------------- */
-nat
+uint32_t
 initHeapProfiling(void)
 {
     if (! RtsFlags.ProfFlags.doHeapProfile) {
@@ -386,7 +401,7 @@ initHeapProfiling(void)
     }
 #ifdef THREADED_RTS
     // See Trac #12019.
-    if (doingLDVProfiling() && RtsFlags.ParFlags.nNodes > 1) {
+    if (doingLDVProfiling() && RtsFlags.ParFlags.nCapabilities > 1) {
         errorBelch("-hb cannot be used with multiple capabilities");
         stg_exit(EXIT_FAILURE);
     }
@@ -405,7 +420,7 @@ initHeapProfiling(void)
     }
 
     // max_era = 2^LDV_SHIFT
-        max_era = 1 << LDV_SHIFT;
+    max_era = 1 << LDV_SHIFT;
 
     n_censuses = 32;
     censuses = stgMallocBytes(sizeof(Census) * n_censuses, "initHeapProfiling");
@@ -433,8 +448,8 @@ initHeapProfiling(void)
     fprintf(hp_file, "SAMPLE_UNIT \"seconds\"\n");
     fprintf(hp_file, "VALUE_UNIT \"bytes\"\n");
 
-    printSample(rtsTrue, 0);
-    printSample(rtsFalse, 0);
+    printSample(true, 0);
+    printSample(false, 0);
 
 #ifdef PROFILING
     if (doingRetainerProfiling()) {
@@ -442,6 +457,9 @@ initHeapProfiling(void)
     }
 #endif
 
+    traceHeapProfBegin(0);
+    dumpCostCentresToEventLog();
+
     return 0;
 }
 
@@ -462,7 +480,7 @@ endHeapProfiling(void)
 
 #ifdef PROFILING
     if (doingLDVProfiling()) {
-        nat t;
+        uint32_t t;
         LdvCensusKillAll();
         aggregateCensusInfo();
         for (t = 1; t < era; t++) {
@@ -473,7 +491,7 @@ endHeapProfiling(void)
 
 #ifdef PROFILING
     if (doingLDVProfiling()) {
-        nat t;
+        uint32_t t;
         if (RtsFlags.ProfFlags.bioSelector != NULL) {
             for (t = 1; t <= era; t++) {
                 freeEra( &censuses[t] );
@@ -491,8 +509,8 @@ endHeapProfiling(void)
     stgFree(censuses);
 
     seconds = mut_user_time();
-    printSample(rtsTrue, seconds);
-    printSample(rtsFalse, seconds);
+    printSample(true, seconds);
+    printSample(false, seconds);
     fclose(hp_file);
 }
 
@@ -512,7 +530,7 @@ buf_append(char *p, const char *q, char *end)
 }
 
 static void
-fprint_ccs(FILE *fp, CostCentreStack *ccs, nat max_length)
+fprint_ccs(FILE *fp, CostCentreStack *ccs, uint32_t max_length)
 {
     char buf[max_length+1], *p, *buf_end;
 
@@ -551,7 +569,7 @@ fprint_ccs(FILE *fp, CostCentreStack *ccs, nat max_length)
     fprintf(fp, "%s", buf);
 }
 
-rtsBool
+bool
 strMatchesSelector( const char* str, const char* sel )
 {
    const char* p;
@@ -564,14 +582,14 @@ strMatchesSelector( const char* str, const char* sel )
        }
        // Match if all of str used and have reached the end of a sel fragment.
        if (*p == '\0' && (*sel == ',' || *sel == '\0'))
-           return rtsTrue;
+           return true;
 
        // No match.  Advance sel to the start of the next elem.
        while (*sel != ',' && *sel != '\0') sel++;
        if (*sel == ',') sel++;
 
        /* Run out of sel ?? */
-       if (*sel == '\0') return rtsFalse;
+       if (*sel == '\0') return false;
    }
 }
 
@@ -581,35 +599,35 @@ strMatchesSelector( const char* str, const char* sel )
  * Figure out whether a closure should be counted in this census, by
  * testing against all the specified constraints.
  * -------------------------------------------------------------------------- */
-static rtsBool
-closureSatisfiesConstraints( StgClosure* p )
+static bool
+closureSatisfiesConstraints( const StgClosure* p )
 {
 #if !defined(PROFILING)
     (void)p;   /* keep gcc -Wall happy */
-    return rtsTrue;
+    return true;
 #else
-   rtsBool b;
+   bool b;
 
    // The CCS has a selected field to indicate whether this closure is
    // deselected by not being mentioned in the module, CC, or CCS
    // selectors.
    if (!p->header.prof.ccs->selected) {
-       return rtsFalse;
+       return false;
    }
 
    if (RtsFlags.ProfFlags.descrSelector) {
        b = strMatchesSelector( (GET_PROF_DESC(get_itbl((StgClosure *)p))),
                                  RtsFlags.ProfFlags.descrSelector );
-       if (!b) return rtsFalse;
+       if (!b) return false;
    }
    if (RtsFlags.ProfFlags.typeSelector) {
        b = strMatchesSelector( (GET_PROF_TYPE(get_itbl((StgClosure *)p))),
                                 RtsFlags.ProfFlags.typeSelector );
-       if (!b) return rtsFalse;
+       if (!b) return false;
    }
    if (RtsFlags.ProfFlags.retainerSelector) {
        RetainerSet *rs;
-       nat i;
+       uint32_t i;
        // We must check that the retainer set is valid here.  One
        // reason it might not be valid is if this closure is a
        // a newly deceased weak pointer (i.e. a DEAD_WEAK), since
@@ -620,13 +638,13 @@ closureSatisfiesConstraints( StgClosure* p )
                for (i = 0; i < rs->num; i++) {
                    b = strMatchesSelector( rs->element[i]->cc->label,
                                            RtsFlags.ProfFlags.retainerSelector );
-                   if (b) return rtsTrue;
+                   if (b) return true;
                }
            }
        }
-       return rtsFalse;
+       return false;
    }
-   return rtsTrue;
+   return true;
 #endif /* PROFILING */
 }
 
@@ -638,7 +656,7 @@ static void
 aggregateCensusInfo( void )
 {
     HashTable *acc;
-    nat t;
+    uint32_t t;
     counter *c, *d, *ctrs;
     Arena *arena;
 
@@ -746,22 +764,29 @@ static void
 dumpCensus( Census *census )
 {
     counter *ctr;
-    long count;
+    ssize_t count;
 
-    printSample(rtsTrue, census->time);
+    printSample(true, census->time);
+    traceHeapProfSampleBegin(era);
 
 #ifdef PROFILING
+    /* change typecast to uint64_t to remove
+     * print formatting warning. See #12636 */
     if (RtsFlags.ProfFlags.doHeapProfile == HEAP_BY_LDV) {
-      fprintf(hp_file, "VOID\t%lu\n", (unsigned long)(census->void_total) * sizeof(W_));
-        fprintf(hp_file, "LAG\t%lu\n",
-                (unsigned long)(census->not_used - census->void_total) * sizeof(W_));
-        fprintf(hp_file, "USE\t%lu\n",
-                (unsigned long)(census->used - census->drag_total) * sizeof(W_));
-        fprintf(hp_file, "INHERENT_USE\t%lu\n",
-                (unsigned long)(census->prim) * sizeof(W_));
-        fprintf(hp_file, "DRAG\t%lu\n",
-                (unsigned long)(census->drag_total) * sizeof(W_));
-        printSample(rtsFalse, census->time);
+        fprintf(hp_file, "VOID\t%" FMT_Word64 "\n",
+                (uint64_t)(census->void_total *
+                                     sizeof(W_)));
+        fprintf(hp_file, "LAG\t%" FMT_Word64 "\n",
+                (uint64_t)((census->not_used - census->void_total) *
+                                     sizeof(W_)));
+        fprintf(hp_file, "USE\t%" FMT_Word64 "\n",
+                (uint64_t)((census->used - census->drag_total) *
+                                     sizeof(W_)));
+        fprintf(hp_file, "INHERENT_USE\t%" FMT_Word64 "\n",
+                (uint64_t)(census->prim * sizeof(W_)));
+        fprintf(hp_file, "DRAG\t%" FMT_Word64 "\n",
+                (uint64_t)(census->drag_total * sizeof(W_)));
+        printSample(false, census->time);
         return;
     }
 #endif
@@ -793,6 +818,8 @@ dumpCensus( Census *census )
         switch (RtsFlags.ProfFlags.doHeapProfile) {
         case HEAP_BY_CLOSURE_TYPE:
             fprintf(hp_file, "%s", (char *)ctr->identity);
+            traceHeapProfSampleString(0, (char *)ctr->identity,
+                                      count * sizeof(W_));
             break;
         }
 #endif
@@ -800,12 +827,17 @@ dumpCensus( Census *census )
 #ifdef PROFILING
         switch (RtsFlags.ProfFlags.doHeapProfile) {
         case HEAP_BY_CCS:
-            fprint_ccs(hp_file, (CostCentreStack *)ctr->identity, RtsFlags.ProfFlags.ccsLength);
+            fprint_ccs(hp_file, (CostCentreStack *)ctr->identity,
+                       RtsFlags.ProfFlags.ccsLength);
+            traceHeapProfSampleCostCentre(0, (CostCentreStack *)ctr->identity,
+                                          count * sizeof(W_));
             break;
         case HEAP_BY_MOD:
         case HEAP_BY_DESCR:
         case HEAP_BY_TYPE:
             fprintf(hp_file, "%s", (char *)ctr->identity);
+            traceHeapProfSampleString(0, (char *)ctr->identity,
+                                      count * sizeof(W_));
             break;
         case HEAP_BY_RETAINER:
         {
@@ -835,22 +867,22 @@ dumpCensus( Census *census )
         }
 #endif
 
-        fprintf(hp_file, "\t%" FMT_SizeT "\n", (W_)count * sizeof(W_));
+        fprintf(hp_file, "\t%" FMT_Word "\n", (W_)count * sizeof(W_));
     }
 
-    printSample(rtsFalse, census->time);
+    printSample(false, census->time);
 }
 
 
-static void heapProfObject(Census *census, StgClosure *p, nat size,
-                           rtsBool prim
+static void heapProfObject(Census *census, StgClosure *p, size_t size,
+                           bool prim
 #ifndef PROFILING
                            STG_UNUSED
 #endif
                            )
 {
-    void *identity;
-    nat real_size;
+    const void *identity;
+    size_t real_size;
     counter *ctr;
 
             identity = NULL;
@@ -877,7 +909,7 @@ static void heapProfObject(Census *census, StgClosure *p, nat size,
                     identity = closureIdentity((StgClosure *)p);
 
                     if (identity != NULL) {
-                        ctr = lookupHashTable( census->hash, (StgWord)identity );
+                        ctr = lookupHashTable(census->hash, (StgWord)identity);
                         if (ctr != NULL) {
 #ifdef PROFILING
                             if (RtsFlags.ProfFlags.bioSelector != NULL) {
@@ -919,6 +951,24 @@ static void heapProfObject(Census *census, StgClosure *p, nat size,
             }
 }
 
+// Compact objects require special handling code because they
+// are not stored consecutively in memory (rather, each object
+// is a list of objects), and that would break the while loop
+// below. But we know that each block holds at most one object
+// so we don't need the loop.
+//
+// See Note [Compact Normal Forms] for details.
+static void
+heapCensusCompactList(Census *census, bdescr *bd)
+{
+    for (; bd != NULL; bd = bd->link) {
+        StgCompactNFDataBlock *block = (StgCompactNFDataBlock*)bd->start;
+        StgCompactNFData *str = block->owner;
+        heapProfObject(census, (StgClosure*)str,
+                       compact_nfdata_full_sizeW(str), true);
+    }
+}
+
 /* -----------------------------------------------------------------------------
  * Code to perform a heap census.
  * -------------------------------------------------------------------------- */
@@ -926,9 +976,9 @@ static void
 heapCensusChain( Census *census, bdescr *bd )
 {
     StgPtr p;
-    StgInfoTable *info;
-    nat size;
-    rtsBool prim;
+    const StgInfoTable *info;
+    size_t size;
+    bool prim;
 
     for (; bd != NULL; bd = bd->link) {
 
@@ -939,7 +989,7 @@ heapCensusChain( Census *census, bdescr *bd )
         if (bd->flags & BF_PINNED) {
             StgClosure arr;
             SET_HDR(&arr, &stg_ARR_WORDS_info, CCS_PINNED);
-            heapProfObject(census, &arr, bd->blocks * BLOCK_SIZE_W, rtsTrue);
+            heapProfObject(census, &arr, bd->blocks * BLOCK_SIZE_W, true);
             continue;
         }
 
@@ -953,14 +1003,14 @@ heapCensusChain( Census *census, bdescr *bd )
         if (bd->flags & BF_LARGE
             && get_itbl((StgClosure *)p)->type == ARR_WORDS) {
             size = arr_words_sizeW((StgArrBytes *)p);
-            prim = rtsTrue;
+            prim = true;
             heapProfObject(census, (StgClosure *)p, size, prim);
             continue;
         }
 
         while (p < bd->free) {
-            info = get_itbl((StgClosure *)p);
-            prim = rtsFalse;
+            info = get_itbl((const StgClosure *)p);
+            prim = false;
 
             switch (info->type) {
 
@@ -980,9 +1030,7 @@ heapCensusChain( Census *census, bdescr *bd )
                 size = sizeofW(StgThunkHeader) + 1;
                 break;
 
-            case CONSTR:
             case FUN:
-            case IND_PERM:
             case BLACKHOLE:
             case BLOCKING_QUEUE:
             case FUN_1_0:
@@ -990,6 +1038,8 @@ heapCensusChain( Census *census, bdescr *bd )
             case FUN_1_1:
             case FUN_0_2:
             case FUN_2_0:
+            case CONSTR:
+            case CONSTR_NOCAF:
             case CONSTR_1_0:
             case CONSTR_0_1:
             case CONSTR_1_1:
@@ -1010,7 +1060,7 @@ heapCensusChain( Census *census, bdescr *bd )
                 break;
 
             case BCO:
-                prim = rtsTrue;
+                prim = true;
                 size = bco_sizeW((StgBCO *)p);
                 break;
 
@@ -1022,7 +1072,7 @@ heapCensusChain( Census *census, bdescr *bd )
             case MUT_PRIM:
             case MUT_VAR_CLEAN:
             case MUT_VAR_DIRTY:
-                prim = rtsTrue;
+                prim = true;
                 size = sizeW_fromITBL(info);
                 break;
 
@@ -1039,7 +1089,7 @@ heapCensusChain( Census *census, bdescr *bd )
                 break;
 
             case ARR_WORDS:
-                prim = rtsTrue;
+                prim = true;
                 size = arr_words_sizeW((StgArrBytes*)p);
                 break;
 
@@ -1047,7 +1097,7 @@ heapCensusChain( Census *census, bdescr *bd )
             case MUT_ARR_PTRS_DIRTY:
             case MUT_ARR_PTRS_FROZEN:
             case MUT_ARR_PTRS_FROZEN0:
-                prim = rtsTrue;
+                prim = true;
                 size = mut_arr_ptrs_sizeW((StgMutArrPtrs *)p);
                 break;
 
@@ -1055,12 +1105,12 @@ heapCensusChain( Census *census, bdescr *bd )
             case SMALL_MUT_ARR_PTRS_DIRTY:
             case SMALL_MUT_ARR_PTRS_FROZEN:
             case SMALL_MUT_ARR_PTRS_FROZEN0:
-                prim = rtsTrue;
+                prim = true;
                 size = small_mut_arr_ptrs_sizeW((StgSmallMutArrPtrs *)p);
                 break;
 
             case TSO:
-                prim = rtsTrue;
+                prim = true;
 #ifdef PROFILING
                 if (RtsFlags.ProfFlags.includeTSOs) {
                     size = sizeofW(StgTSO);
@@ -1076,7 +1126,7 @@ heapCensusChain( Census *census, bdescr *bd )
 #endif
 
             case STACK:
-                prim = rtsTrue;
+                prim = true;
 #ifdef PROFILING
                 if (RtsFlags.ProfFlags.includeTSOs) {
                     size = stack_sizeW((StgStack*)p);
@@ -1092,10 +1142,14 @@ heapCensusChain( Census *census, bdescr *bd )
 #endif
 
             case TREC_CHUNK:
-                prim = rtsTrue;
+                prim = true;
                 size = sizeofW(StgTRecChunk);
                 break;
 
+            case COMPACT_NFDATA:
+                barf("heapCensus, found compact object in the wrong list");
+                break;
+
             default:
                 barf("heapCensus, unknown object: %d", info->type);
             }
@@ -1109,7 +1163,7 @@ heapCensusChain( Census *census, bdescr *bd )
 
 void heapCensus (Time t)
 {
-  nat g, n;
+  uint32_t g, n;
   Census *census;
   gen_workspace *ws;
 
@@ -1133,6 +1187,7 @@ void heapCensus (Time t)
       // Are we interested in large objects?  might be
       // confusing to include the stack in a heap profile.
       heapCensusChain( census, generations[g].large_objects );
+      heapCensusCompactList ( census, generations[g].compact_objects );
 
       for (n = 0; n < n_capabilities; n++) {
           ws = &gc_threads[n]->gens[g];
