@@ -38,10 +38,10 @@ INLINE_HEADER StgClosure* reclaimSpark(SparkPool *pool);
 
 // Returns True if the spark pool is empty (can give a false positive
 // if the pool is almost empty).
-INLINE_HEADER rtsBool looksEmpty(SparkPool* deque);
+INLINE_HEADER bool looksEmpty(SparkPool* deque);
 
 INLINE_HEADER StgClosure * tryStealSpark (SparkPool *pool);
-INLINE_HEADER rtsBool      fizzledSpark  (StgClosure *);
+INLINE_HEADER bool         fizzledSpark  (StgClosure *);
 
 void         freeSparkPool     (SparkPool *pool);
 void         createSparkThread (Capability *cap);
@@ -60,7 +60,7 @@ INLINE_HEADER StgClosure* reclaimSpark(SparkPool *pool)
     return popWSDeque(pool);
 }
 
-INLINE_HEADER rtsBool looksEmpty(SparkPool* deque)
+INLINE_HEADER bool looksEmpty(SparkPool* deque)
 {
     return looksEmptyWSDeque(deque);
 }
@@ -96,7 +96,7 @@ INLINE_HEADER StgClosure * tryStealSpark (SparkPool *pool)
     // other pools before trying again.
 }
 
-INLINE_HEADER rtsBool fizzledSpark (StgClosure *spark)
+INLINE_HEADER bool fizzledSpark (StgClosure *spark)
 {
     return (GET_CLOSURE_TAG(spark) != 0 || !closure_SHOULD_SPARK(spark));
 }
