@@ -39,7 +39,7 @@ runAllCFinalizers(StgWeak *list)
 
     task = myTask();
     if (task != NULL) {
-        task->running_finalizers = rtsTrue;
+        task->running_finalizers = true;
     }
 
     for (w = list; w; w = w->link) {
@@ -56,7 +56,7 @@ runAllCFinalizers(StgWeak *list)
     }
 
     if (task != NULL) {
-        task->running_finalizers = rtsFalse;
+        task->running_finalizers = false;
     }
 }
 
@@ -82,12 +82,12 @@ scheduleFinalizers(Capability *cap, StgWeak *list)
     StgTSO *t;
     StgMutArrPtrs *arr;
     StgWord size;
-    nat n, i;
+    uint32_t n, i;
     Task *task;
 
     task = myTask();
     if (task != NULL) {
-        task->running_finalizers = rtsTrue;
+        task->running_finalizers = true;
     }
 
     // count number of finalizers, and kill all the weak pointers first...
@@ -110,13 +110,13 @@ scheduleFinalizers(Capability *cap, StgWeak *list)
         // Furthermore, when PROFILING is turned on, dead weak
         // pointers are exactly as large as weak pointers, so there is
         // no need to fill the slop, either.  See stg_DEAD_WEAK_info
-        // in StgMiscClosures.hc.
+        // in StgMiscClosures.cmm.
 #endif
         SET_HDR(w, &stg_DEAD_WEAK_info, w->header.prof.ccs);
     }
 
     if (task != NULL) {
-        task->running_finalizers = rtsFalse;
+        task->running_finalizers = false;
     }
 
     // No finalizers to run?
