@@ -21,7 +21,7 @@ extern char *ghc_rts_opts;
 extern void OnExitHook (void);
 extern void StackOverflowHook (W_ stack_size);
 extern void OutOfHeapHook (W_ request_size, W_ heap_size);
-extern void MallocFailHook (W_ request_size /* in bytes */, char *msg);
+extern void MallocFailHook (W_ request_size /* in bytes */, const char *msg);
 extern void FlagDefaultsHook (void);
 
 #include "EndPrivate.h"
