@@ -13,9 +13,11 @@
 # include <signal.h>
 #endif
 
+#include "Ticker.h"
+
 #include "BeginPrivate.h"
 
-rtsBool anyUserHandlers(void);
+bool anyUserHandlers(void);
 
 #if !defined(THREADED_RTS)
 extern siginfo_t pending_handler_buf[];
@@ -24,6 +26,8 @@ extern siginfo_t *next_pending_handler;
 void startSignalHandlers(Capability *cap);
 #endif
 
+void install_vtalrm_handler(int sig, TickProc handle_tick);
+
 void ioManagerStartCap (/* inout */ Capability **cap);
 
 extern StgInt *signal_handlers;
