@@ -18,22 +18,34 @@
 
 #include "GCTDecl.h"
 
-bdescr *allocBlock_sync(void);
+bdescr* allocGroup_sync(uint32_t n);
+bdescr* allocGroupOnNode_sync(uint32_t node, uint32_t n);
+
+INLINE_HEADER bdescr *allocBlock_sync(void)
+{
+    return allocGroup_sync(1);
+}
+
+INLINE_HEADER bdescr *allocBlockOnNode_sync(uint32_t node)
+{
+    return allocGroupOnNode_sync(node,1);
+}
+
 void    freeChain_sync(bdescr *bd);
 
 void    push_scanned_block   (bdescr *bd, gen_workspace *ws);
-StgPtr  todo_block_full      (nat size, gen_workspace *ws);
-StgPtr  alloc_todo_block     (gen_workspace *ws, nat size);
+StgPtr  todo_block_full      (uint32_t size, gen_workspace *ws);
+StgPtr  alloc_todo_block     (gen_workspace *ws, uint32_t size);
 
 bdescr *grab_local_todo_block  (gen_workspace *ws);
 #if defined(THREADED_RTS)
-bdescr *steal_todo_block       (nat s);
+bdescr *steal_todo_block       (uint32_t s);
 #endif
 
 // Returns true if a block is partially full.  This predicate is used to try
 // to re-use partial blocks wherever possible, and to reduce wastage.
 // We might need to tweak the actual value.
-INLINE_HEADER rtsBool
+INLINE_HEADER bool
 isPartiallyFull(bdescr *bd)
 {
     return (bd->free + WORK_UNIT_WORDS < bd->start + BLOCK_SIZE_W);
@@ -48,7 +60,7 @@ void printMutableList (bdescr *bd);
 // mutable lists attached to the current gc_thread structure, which
 // are the same as the mutable lists on the Capability.
 INLINE_HEADER void
-recordMutableGen_GC (StgClosure *p, nat gen_no)
+recordMutableGen_GC (StgClosure *p, uint32_t gen_no)
 {
     bdescr *bd;
 
