@@ -2,7 +2,7 @@
  *
  * (c) The GHC Team, 1998-2009
  *
- * External Storage Manger Interface
+ * Storage Manger Interface
  *
  * ---------------------------------------------------------------------------*/
 
@@ -19,41 +19,27 @@
 
 void initStorage(void);
 void exitStorage(void);
-void freeStorage(rtsBool free_heap);
+void freeStorage(bool free_heap);
 
 // Adding more Capabilities later: this function allocates nurseries
 // and initialises other storage-related things.
-void storageAddCapabilities (nat from, nat to);
+void storageAddCapabilities (uint32_t from, uint32_t to);
 
 /* -----------------------------------------------------------------------------
-   Storage manager state
+   Should we GC?
    -------------------------------------------------------------------------- */
 
-INLINE_HEADER rtsBool
-doYouWantToGC( Capability *cap )
+INLINE_HEADER
+bool doYouWantToGC(Capability *cap)
 {
-  return (cap->r.rCurrentNursery->link == NULL ||
-          g0->n_new_large_words >= large_alloc_lim);
+    return (cap->r.rCurrentNursery->link == NULL ||
+            g0->n_new_large_words >= large_alloc_lim);
 }
 
-/* for splitting blocks groups in two */
-bdescr * splitLargeBlock (bdescr *bd, W_ blocks);
-
 /* -----------------------------------------------------------------------------
-   Generational garbage collection support
-
-   updateWithIndirection(p1,p2)  Updates the object at p1 with an
-                                 indirection pointing to p2.  This is
-                                 normally called for objects in an old
-                                 generation (>0) when they are updated.
-
-   updateWithPermIndirection(p1,p2)  As above but uses a permanent indir.
-
+   The storage manager mutex
    -------------------------------------------------------------------------- */
 
-/*
- * Storage manager mutex
- */
 #if defined(THREADED_RTS)
 extern Mutex sm_mutex;
 #endif
@@ -80,14 +66,14 @@ void dirty_TVAR(Capability *cap, StgTVar *p);
    -------------------------------------------------------------------------- */
 
 extern nursery *nurseries;
-extern nat n_nurseries;
+extern uint32_t n_nurseries;
 
-void     resetNurseries       ( void );
-void     clearNursery         ( Capability *cap );
-void     resizeNurseries      ( W_ blocks );
-void     resizeNurseriesFixed ( void );
-W_       countNurseryBlocks   ( void );
-rtsBool  getNewNursery        ( Capability *cap );
+void     resetNurseries       (void);
+void     clearNursery         (Capability *cap);
+void     resizeNurseries      (StgWord blocks);
+void     resizeNurseriesFixed (void);
+StgWord  countNurseryBlocks   (void);
+bool     getNewNursery        (Capability *cap);
 
 /* -----------------------------------------------------------------------------
    Allocation accounting
@@ -114,18 +100,17 @@ StgWord calcTotalAllocated   (void);
    Stats 'n' DEBUG stuff
    -------------------------------------------------------------------------- */
 
-W_    countLargeAllocated  (void);
-W_    countOccupied        (bdescr *bd);
-W_    calcNeeded           (rtsBool force_major, W_ *blocks_needed);
+StgWord countOccupied       (bdescr *bd);
+StgWord calcNeeded          (bool force_major, StgWord *blocks_needed);
 
-W_    gcThreadLiveWords  (nat i, nat g);
-W_    gcThreadLiveBlocks (nat i, nat g);
+StgWord gcThreadLiveWords  (uint32_t i, uint32_t g);
+StgWord gcThreadLiveBlocks (uint32_t i, uint32_t g);
 
-W_    genLiveWords  (generation *gen);
-W_    genLiveBlocks (generation *gen);
+StgWord genLiveWords  (generation *gen);
+StgWord genLiveBlocks (generation *gen);
 
-W_    calcLiveBlocks (void);
-W_    calcLiveWords  (void);
+StgWord calcTotalLargeObjectsW (void);
+StgWord calcTotalCompactW (void);
 
 /* ----------------------------------------------------------------------------
    Storage manager internal APIs and globals
@@ -133,7 +118,7 @@ W_    calcLiveWords  (void);
 
 extern bdescr *exec_block;
 
-void move_STACK  (StgStack *src, StgStack *dest);
+void move_STACK (StgStack *src, StgStack *dest);
 
 /* -----------------------------------------------------------------------------
    Note [STATIC_LINK fields]
@@ -170,7 +155,7 @@ void move_STACK  (StgStack *src, StgStack *dest);
 
 // The previous and current values of the static flag.  These flip
 // between STATIC_FLAG_A and STATIC_FLAG_B at each major GC.
-extern nat prev_static_flag, static_flag;
+extern uint32_t prev_static_flag, static_flag;
 
 // In the chain of static objects built up during GC, all the link
 // fields are tagged with the current static_flag value.  How to mark
