@@ -11,9 +11,7 @@
 #include "sm/HeapAlloc.h"
 #include "RtsUtils.h"
 
-#if HAVE_WINDOWS_H
 #include <windows.h>
-#endif
 
 typedef struct alloc_rec_ {
     char* base;    // non-aligned base address, directly from VirtualAlloc
@@ -39,16 +37,33 @@ static alloc_rec* allocs = NULL;
 /* free_blocks are kept in ascending order, and adjacent blocks are merged */
 static block_rec* free_blocks = NULL;
 
+/* Mingw-w64 does not currently have this in their header. So we have to import it.*/
+typedef LPVOID(WINAPI *VirtualAllocExNumaProc)(HANDLE, LPVOID, SIZE_T, DWORD, DWORD, DWORD);
+
+/* Cache NUMA API call. */
+VirtualAllocExNumaProc VirtualAllocExNuma;
+
 void
 osMemInit(void)
 {
     allocs = NULL;
     free_blocks = NULL;
+
+    /* Resolve and cache VirtualAllocExNuma. */
+    if (osNumaAvailable() && RtsFlags.GcFlags.numa)
+    {
+        VirtualAllocExNuma = (VirtualAllocExNumaProc)GetProcAddress(GetModuleHandleW(L"kernel32"), "VirtualAllocExNuma");
+        if (!VirtualAllocExNuma)
+        {
+            sysErrorBelch(
+                "osBindMBlocksToNode: VirtualAllocExNuma does not exist. How did you get this far?");
+        }
+    }
 }
 
 static
 alloc_rec*
-allocNew(nat n) {
+allocNew(uint32_t n) {
     alloc_rec* rec;
     rec = (alloc_rec*)stgMallocBytes(sizeof(alloc_rec),"getMBlocks: allocNew");
     rec->size = ((W_)n+1)*MBLOCK_SIZE;
@@ -59,7 +74,7 @@ allocNew(nat n) {
         rec=0;
         if (GetLastError() == ERROR_NOT_ENOUGH_MEMORY) {
 
-            errorBelch("Out of memory");
+            errorBelch("Out of memory\n");
             stg_exit(EXIT_HEAPOVERFLOW);
         } else {
             sysErrorBelch(
@@ -117,7 +132,7 @@ insertFree(char* alloc_base, W_ alloc_size) {
 
 static
 void*
-findFreeBlocks(nat n) {
+findFreeBlocks(uint32_t n) {
     void* ret=0;
     block_rec* it;
     block_rec temp;
@@ -186,7 +201,7 @@ commitBlocks(char* base, W_ size) {
 }
 
 void *
-osGetMBlocks(nat n) {
+osGetMBlocks(uint32_t n) {
     void* ret;
     ret = findFreeBlocks(n);
     if(ret==0) {
@@ -246,7 +261,7 @@ static void decommitBlocks(char *addr, W_ nBytes)
     }
 }
 
-void osFreeMBlocks(char *addr, nat n)
+void osFreeMBlocks(void *addr, uint32_t n)
 {
     W_ nBytes = (W_)n * MBLOCK_SIZE;
 
@@ -381,17 +396,17 @@ osFreeAllMBlocks(void)
     }
 }
 
-W_ getPageSize (void)
+size_t getPageSize (void)
 {
-    static W_ pagesize = 0;
-    if (pagesize) {
-        return pagesize;
-    } else {
+    static size_t pagesize = 0;
+
+    if (pagesize == 0) {
         SYSTEM_INFO sSysInfo;
         GetSystemInfo(&sSysInfo);
         pagesize = sSysInfo.dwPageSize;
-        return pagesize;
     }
+
+    return pagesize;
 }
 
 /* Returns 0 if physical memory size cannot be identified */
@@ -413,7 +428,7 @@ StgWord64 getPhysicalMemorySize (void)
     return physMemSize;
 }
 
-void setExecutable (void *p, W_ len, rtsBool exec)
+void setExecutable (void *p, W_ len, bool exec)
 {
     DWORD dwOldProtect = 0;
     if (VirtualProtect (p, len,
@@ -483,3 +498,75 @@ void osReleaseHeapMemory (void)
 }
 
 #endif
+
+bool osNumaAvailable(void)
+{
+    return osNumaNodes() > 1;
+}
+
+uint32_t osNumaNodes(void)
+{
+    /* Cache the amount of NUMA values. */
+    static ULONG numNumaNodes = 0;
+
+    /* Cache the amount of NUMA nodes. */
+    if (!numNumaNodes && !GetNumaHighestNodeNumber(&numNumaNodes))
+    {
+        numNumaNodes = 1;
+    }
+
+    return numNumaNodes;
+}
+
+uint64_t osNumaMask(void)
+{
+    uint64_t numaMask;
+    if (!GetNumaNodeProcessorMask(0, &numaMask))
+    {
+        return 1;
+    }
+    return numaMask;
+}
+
+void osBindMBlocksToNode(
+    void *addr,
+    StgWord size,
+    uint32_t node)
+{
+    if (osNumaAvailable())
+    {
+        void* temp;
+        if (RtsFlags.GcFlags.numa) {
+            /* Note [base memory]
+               I would like to use addr here to specify the base
+               memory of allocation. The problem is that the address
+               we are requesting is too high. I can't figure out if it's
+               because of my NUMA-emulation or a bug in the code.
+
+               On windows also -xb is broken, it does nothing so that can't
+               be used to tweak it (see #12577). So for now, just let the OS decide.
+            */
+            temp = VirtualAllocExNuma(
+                          GetCurrentProcess(),
+                          NULL, // addr? See base memory
+                          size,
+                          MEM_RESERVE | MEM_COMMIT,
+                          PAGE_READWRITE,
+                          node
+                        );
+
+            if (!temp) {
+                if (GetLastError() == ERROR_NOT_ENOUGH_MEMORY) {
+                    errorBelch("out of memory");
+                }
+                else {
+                    sysErrorBelch(
+                        "osBindMBlocksToNode: VirtualAllocExNuma MEM_RESERVE %" FMT_Word " bytes "
+                        "at address %p bytes failed",
+                                        size, addr);
+                }
+                stg_exit(EXIT_FAILURE);
+            }
+        }
+    }
+}
