@@ -106,7 +106,6 @@ loadRegOffs = vcat . map (uncurry assign_stk_to_reg)
 saveRegOffs :: [(Reg,Int)] -> Doc
 saveRegOffs = vcat . map (uncurry assign_reg_to_stk)
 
--- a bit like assignRegs in CgRetConv.lhs
 assignRegs
         :: RegStatus            -- are we registerised?
         -> Int                  -- Sp of first arg
@@ -332,7 +331,7 @@ genMkPAP regstatus macro jump live ticker disamb
 
                 -- for a PAP, we have to arrange that the stack contains a
                 -- return address in the event that stg_PAP_entry fails its
-                -- heap check.  See stg_PAP_entry in Apply.hc for details.
+                -- heap check.  See stg_PAP_entry in Apply.cmm for details.
              if is_pap
                 then text "R2 = " <> mkApplyInfoName this_call_args <> semi
 
@@ -524,7 +523,7 @@ enterFastPath regstatus no_load_regs args_in_regs args
     = enterFastPathHelper tag regstatus no_load_regs args_in_regs args
 enterFastPath _ _ _ _ = empty
 
--- Copied from Constants.lhs & CgUtils.hs, i'd rather have this imported:
+-- Copied from Constants.hs & CgUtils.hs, i'd rather have this imported:
 -- (arity,tag)
 tAG_BITS = (TAG_BITS :: Int)
 tAG_BITS_MAX = ((1 `shiftL` tAG_BITS) :: Int)
@@ -655,7 +654,6 @@ genApply regstatus args =
 --        print "    [THUNK_SELECTOR]  &&thunk_lbl,"
 --        print "    [IND]            &&ind_lbl,"
 --        print "    [IND_STATIC]      &&ind_lbl,"
---        print "    [IND_PERM]       &&ind_lbl,"
 --        print "  };"
 
        tickForArity (length args),
@@ -774,8 +772,7 @@ genApply regstatus args =
 --        print "    ind_lbl:"
 --    else:
         text "case IND,",
-        text "     IND_STATIC,",
-        text "     IND_PERM: {",
+        text "     IND_STATIC: {",
         nest 4 (vcat [
           text "R1 = StgInd_indirectee(R1);",
             -- An indirection node might contain a tagged pointer
@@ -905,7 +902,7 @@ genStackApply regstatus args =
 -- These code fragments are used to save registers on the stack at a heap
 -- check failure in the entry code for a function.  We also have to save R1
 -- and the return address (stg_gc_fun_info) on the stack.  See stg_gc_fun_gen
--- in HeapStackCheck.hc for more details.
+-- in HeapStackCheck.cmm for more details.
 
 mkStackSaveEntryLabel :: [ArgRep] -> Doc
 mkStackSaveEntryLabel args = text "stg_stk_save_" <> text (concatMap showArg args)
@@ -927,7 +924,8 @@ genStackSave regstatus args =
                 ]
 
    std_frame_size = 3 -- the std bits of the frame. See StgRetFun in Closures.h,
-                      -- and the comment on stg_fun_gc_gen in HeapStackCheck.hc.
+                      -- and the comment on stg_fun_gc_gen
+                      -- in HeapStackCheck.cmm.
    (reg_locs, leftovers, sp_offset) = assignRegs regstatus std_frame_size args
 
    -- number of words of arguments on the stack.
@@ -945,7 +943,7 @@ main = do
                               exitWith (ExitFailure 1)
   let the_code = vcat [
                 text "// DO NOT EDIT!",
-                text "// Automatically generated by GenApply.hs",
+                text "// Automatically generated by utils/genapply/Main.hs",
                 text "",
                 text "#include \"Cmm.h\"",
                 text "#include \"AutoApply.h\"",
@@ -996,7 +994,7 @@ applyTypes = [
 --
 --  NOTE: other places to change if you change stackApplyTypes:
 --       - includes/rts/storage/FunTypes.h
---       - compiler/codeGen/CgCallConv.lhs: stdPattern
+--       - compiler/codeGen/StgCmmLayout.hs: stdPattern
 stackApplyTypes = [
         [],
         [N],
