@@ -8,11 +8,23 @@ keepingIntermediatesOptions =
          , flagDescription = "Retain intermediate ``.hc`` files."
          , flagType = DynamicFlag
          }
+  , flag { flagName = "-keep-hi-files"
+         , flagDescription =
+           "Retain intermediate ``.hi`` files (the default)."
+         , flagType = DynamicFlag
+         , flagReverse = "-no-keep-hi-files"
+         }
   , flag { flagName = "-keep-llvm-file, -keep-llvm-files"
          , flagDescription = "Retain intermediate LLVM ``.ll`` files. "++
            "Implies :ghc-flag:`-fllvm`."
          , flagType = DynamicFlag
          }
+  , flag { flagName = "-keep-o-files"
+         , flagDescription =
+           "Retain intermediate ``.o`` files (the default)."
+         , flagType = DynamicFlag
+         , flagReverse = "-no-keep-o-files"
+         }
   , flag { flagName = "-keep-s-file, -keep-s-files"
          , flagDescription = "Retain intermediate ``.s`` files."
          , flagType = DynamicFlag
