@@ -8,11 +8,16 @@ verbosityOptions =
          , flagDescription = "verbose mode (equivalent to ``-v3``)"
          , flagType = DynamicFlag
          }
-  , flag { flagName = "-v⟨n⟩"
+  , flag { flagName = "-v ⟨n⟩"
          , flagDescription = "set verbosity level"
          , flagType = DynamicFlag
          , flagReverse = ""
          }
+  , flag { flagName = "-fhide-source-paths"
+         , flagDescription = "hide module source and object paths"
+         , flagType = DynamicFlag
+         , flagReverse = ""
+         }
   , flag { flagName = "-fprint-potential-instances"
          , flagDescription =
            "display all available instances in type error messages"
@@ -29,7 +34,7 @@ verbosityOptions =
   , flag { flagName = "-fprint-explicit-kinds"
          , flagDescription =
            "Print explicit kind foralls and kind arguments in types. " ++
-           "See also :ghc-flag:`-XKindSignature`"
+           "See also :ghc-flag:`-XKindSignatures`"
          , flagType = DynamicFlag
          , flagReverse = "-fno-print-explicit-kinds"
          }
@@ -59,6 +64,14 @@ verbosityOptions =
          , flagType = DynamicFlag
          , flagReverse = "-fno-print-typechecker-elaboration"
          }
+  , flag { flagName = "-fdiagnostics-color=(always|auto|never)"
+         , flagDescription = "Use colors in error messages"
+         , flagType = DynamicFlag
+         }
+  , flag { flagName = "-f[no-]diagnostics-show-caret"
+         , flagDescription = "Whether to show snippets of original source code"
+         , flagType = DynamicFlag
+         }
   , flag { flagName = "-ferror-spans"
          , flagDescription = "Output full span in error messages"
          , flagType = DynamicFlag
@@ -68,4 +81,12 @@ verbosityOptions =
            "Summarise timing stats for GHC (same as ``+RTS -tstderr``)."
          , flagType = DynamicFlag
          }
+  , flag { flagName = "-fshow-hole-constraints"
+         , flagDescription = "Show constraints when reporting typed holes"
+         , flagType = DynamicFlag
+         }
+  , flag { flagName = "-fshow-loaded-modules"
+         , flagDescription = "Show the names of modules that GHCi loaded after a :ghci-cmd:`:load` command."
+         , flagType = DynamicFlag
+         }
   ]
